# -*- coding: utf-8 -*-

import sys
import pytest
import numpy as np

from mirdata import track

if sys.version_info.major == 3:
    builtin_module_name = "builtins"
else:
    builtin_module_name = "__builtin__"


def test_track_repr():
    class TestTrack(track.Track):
        def __init__(self):
            self.a = "asdf"
            self.b = 1.2345678
            self.c = {1: "a", "b": 2}
            self._d = "hidden"
            self.e = None
            self.long = "a" + "b" * 50 + "c" * 50

        @property
        def f(self):
            """ThisObjectType: I have a docstring"""
            return None

        @property
        def g(self):
            """I have an improper docstring"""
            return None

        def h(self):
            return "I'm a function!"

    expected1 = """Track(\n  a="asdf",\n  b=1.2345678,\n  """
    expected2 = """c={1: 'a', 'b': 2},\n  e=None,\n  """
    expected3 = """long="...{}",\n  """.format("b" * 50 + "c" * 50)
    expected4 = """f: ThisObjectType,\n  g: I have an improper docstring,\n)"""

    test_track = TestTrack()
    actual = test_track.__repr__()
    assert actual == expected1 + expected2 + expected3 + expected4

    with pytest.raises(NotImplementedError):
        test_track.to_jams()

    class NoDocsTrack(track.Track):
        @property
        def no_doc(self):
            return "whee!"

    bad_track = NoDocsTrack()
    with pytest.raises(ValueError):
        bad_track.__repr__()


def test_multitrack_basic():
    class TestTrack(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (2, 100)), 1000

    class TestMultiTrack1(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home

    mtrack = TestMultiTrack1("test", "foo")

    with pytest.raises(NotImplementedError):
        mtrack.to_jams()

    with pytest.raises(NotImplementedError):
        mtrack.get_target(["a"])

    with pytest.raises(NotImplementedError):
        mtrack.get_random_target()

    with pytest.raises(NotImplementedError):
        mtrack.get_mix()

    class TestMultiTrack2(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

        def to_jams(self):
            return None

    mtrack = TestMultiTrack2("test", "foo")
    mtrack.to_jams()
    mtrack.get_target(["a"])
    mtrack.get_random_target()
    mtrack.get_mix()


def test_multitrack_mixing():
    class TestTrack(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (2, 100)), 1000

    class TestMultiTrack(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

    mtrack = TestMultiTrack("test", "foo")

    target1 = mtrack.get_target(["a", "c"])
    assert target1.shape == (2, 100)
    assert np.max(np.abs(target1)) <= 1

    target2 = mtrack.get_target(["b", "c"], weights=[0.5, 0.2])
    assert target2.shape == (2, 100)
    assert np.max(np.abs(target2)) <= 1

    target3 = mtrack.get_target(["b", "c"], weights=[0.5, 5])
    assert target3.shape == (2, 100)
    assert np.max(np.abs(target3)) <= 1

    target4 = mtrack.get_target(["a", "c"], average=False)
    assert target4.shape == (2, 100)
    assert np.max(np.abs(target4)) <= 2

    target5 = mtrack.get_target(["a", "c"], average=False, weights=[0.1, 0.5])
    assert target5.shape == (2, 100)
    assert np.max(np.abs(target5)) <= 0.6

    random_target1, t1, w1 = mtrack.get_random_target(n_tracks=2)
    assert random_target1.shape == (2, 100)
    assert np.max(np.abs(random_target1)) <= 1
    assert len(t1) == 2
    assert len(w1) == 2
    assert np.all(w1 >= 0.3)
    assert np.all(w1 <= 1.0)

    random_target2, t2, w2 = mtrack.get_random_target(n_tracks=5)
    assert random_target2.shape == (2, 100)
    assert np.max(np.abs(random_target2)) <= 1
    assert len(t2) == 3
    assert len(w2) == 3
    assert np.all(w2 >= 0.3)
    assert np.all(w2 <= 1.0)

    random_target3, t3, w3 = mtrack.get_random_target()
    assert random_target3.shape == (2, 100)
    assert np.max(np.abs(random_target3)) <= 1
    assert len(t3) == 3
    assert len(w3) == 3
    assert np.all(w3 >= 0.3)
    assert np.all(w3 <= 1.0)

    random_target4, t4, w4 = mtrack.get_random_target(
        n_tracks=2, min_weight=0.1, max_weight=0.4
    )
    assert random_target4.shape == (2, 100)
    assert np.max(np.abs(random_target4)) <= 1
    assert len(t4) == 2
    assert len(w4) == 2
    assert np.all(w4 >= 0.1)
    assert np.all(w4 <= 0.4)

    mix = mtrack.get_mix()
    assert mix.shape == (2, 100)


def test_multitrack_unequal_len():
    class TestTrack(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (2, np.random.randint(50, 100))), 1000

    class TestMultiTrack(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

    mtrack = TestMultiTrack("test", "foo")

    with pytest.raises(ValueError):
        mtrack.get_target(["a", "b", "c"])

    target1 = mtrack.get_target(["a", "b", "c"], enforce_length=False)
    assert target1.shape[0] == 2
    assert np.max(np.abs(target1)) <= 1

    target2 = mtrack.get_target(["a", "b", "c"], average=False, enforce_length=False)
    assert target2.shape[0] == 2
    assert np.max(np.abs(target2)) <= 3


def test_multitrack_unequal_sr():
    class TestTrack(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (2, 100)), np.random.randint(10, 1000)

    class TestMultiTrack(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

    mtrack = TestMultiTrack("test", "foo")

    with pytest.raises(ValueError):
        mtrack.get_target(["a", "b", "c"])


def test_multitrack_mono():
    ### no first channel - audio shapes (100,)
    class TestTrack(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (100)), 1000

    class TestMultiTrack(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

    mtrack = TestMultiTrack("test", "foo")

    target1 = mtrack.get_target(["a", "c"])
    assert target1.shape == (1, 100)
    assert np.max(np.abs(target1)) <= 1

    target1 = mtrack.get_target(["a", "c"], average=False)
    assert target1.shape == (1, 100)
    assert np.max(np.abs(target1)) <= 2

    ### one channel mono shape (1, 100)
    class TestTrack1(track.Track):
        def __init__(self, key):
            self.key = key

        @property
        def f(self):
            return np.random.uniform(-1, 1, (1, 100)), 1000

    class TestMultiTrack1(track.MultiTrack):
        def __init__(self, mtrack_id, data_home):
            self.mtrack_id = mtrack_id
            self._data_home = data_home
            self.tracks = {t: TestTrack1(t) for t in ["a", "b", "c"]}
            self.track_audio_property = "f"

    mtrack = TestMultiTrack1("test", "foo")

    target1 = mtrack.get_target(["a", "c"])
    assert target1.shape == (1, 100)
    assert np.max(np.abs(target1)) <= 1

    target1 = mtrack.get_target(["a", "c"], average=False)
    assert target1.shape == (1, 100)
    assert np.max(np.abs(target1)) <= 2



# -*- coding: utf-8 -*-
"""giantsteps_tempo Dataset Loader

name:             GiantSteps (tempo+genre)

contact:          Richard Vogl <richard.vogl@tuwien.ac.at>
                  Peter Knees <peter.knees@tuwien.ac.at>

description:      collection of annotations for 664 2min(1) audio previews from
                  www.beatport.com

references:       [1] Peter Knees, Ángel Faraldo, Perfecto Herrera, Richard Vogl,
                  Sebastian Böck, Florian Hörschläger, Mickael Le Goff: "Two data
                  sets for tempo estimation and key detection in electronic dance
                  music annotated from user corrections", Proc. of the 16th
                  Conference of the International Society for Music Information
                  Retrieval (ISMIR'15), Oct. 2015, Malaga, Spain.

                  [2] Hendrik Schreiber, Meinard Müller: "A Crowdsourced Experiment
                  for Tempo Estimation of Electronic Dance Music", Proc. of the
                  19th Conference of the International Society for Music
                  Information Retrieval (ISMIR'18), Sept. 2018, Paris, France.

annotations:      tempo (bpm), genre

notes:
=========================================================================
The audio files (664 files, size ~1gb) can be downloaded from http://www.beatport.com/
using the bash script:

 https://github.com/GiantSteps/giantsteps-tempo-dataset/blob/master/audio_dl.sh

To download the files manually use links of the following form:
http://geo-samples.beatport.com/lofi/<name of mp3 file>
e.g.:
http://geo-samples.beatport.com/lofi/5377710.LOFI.mp3

To convert the audio files to .wav use (bash + sox):

./convert_audio.sh

To retrieve the genre information, the JSON contained within the website was parsed.
The tempo annotation was extracted from forum entries of people correcting the bpm values (i.e. manual annotation of tempo).
For more information please contact creators.

[2] found some files without tempo. There are:

3041381.LOFI.mp3
3041383.LOFI.mp3
1327052.LOFI.mp3

Their v2 tempo is denoted as 0.0 in tempo and mirex and has no annotation in the JAMS format.

(1): Most of the audio files are 120 seconds long. Exceptions are:
name              length
906760.LOFI.mp3   62
1327052.LOFI.mp3  70
4416506.LOFI.mp3  80
1855660.LOFI.mp3  119
3419452.LOFI.mp3  119
3577631.LOFI.mp3  119
"""


import librosa
import os

from mirdata import download_utils
from mirdata import track
from mirdata import utils
import numpy as np
import jams

DATASET_DIR = 'GiantSteps_tempo'

DATA = utils.LargeData('giantsteps_tempo_index.json')

REMOTES = {
    'annotations': download_utils.RemoteFileMetadata(
        filename='giantsteps-tempo-dataset-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb.zip',
        url='https://github.com/GiantSteps/giantsteps-tempo-dataset/archive/0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb.zip',
        checksum='8fdafbaf505fe3f293bd912c92b72ac8',
        destination_dir='',
    )
}


class Track(track.Track):
    """giantsteps_tempo track class
    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Attributes:
        audio_path (str): track audio path
        title (str): title of the track
        track_id (str): track id
        annotation_v1_path (str): track annotation v1 path
        annotation_v2_path (str): track annotation v2 path
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError(
                '{} is not a valid track ID in giantsteps_tempo'.format(track_id)
            )

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]
        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])
        self.annotation_v1_path = os.path.join(
            self._data_home, self._track_paths['annotation_v1'][0]
        )
        self.annotation_v2_path = os.path.join(
            self._data_home, self._track_paths['annotation_v2'][0]
        )

        self.title = self.audio_path.replace(".mp3", '').split('/')[-1].split('.')[0]

    @utils.cached_property
    def genre(self):
        """genre: human-labeled metadata annotation"""
        return load_genre(self.annotation_v1_path)

    @utils.cached_property
    def tempo(self):
        """TempoData: tempo annotation ordered by confidence"""
        return load_tempo(self.annotation_v1_path)

    @utils.cached_property
    def tempo_v2(self):
        """TempoData: tempos annotation ordered by confidence"""
        return load_tempo(self.annotation_v2_path)

    @property
    def audio(self):
        """(np.ndarray, float): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams.load(self.annotation_v1_path)

    def to_jams_v2(self):
        """Jams: the track's data in jams format"""
        return jams.load(self.annotation_v2_path)


def load_audio(audio_path):
    """Load a giantsteps_tempo audio file.
    Args:
        audio_path (str): path to audio file
    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file
    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))
    return librosa.load(audio_path, sr=None, mono=True)


def download(data_home=None, force_overwrite=False, cleanup=True):
    """Download the giantsteps_tempo Dataset (annotations).
    The audio files are not provided due to copyright issues.
    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.
        partial_download(list of str)
            arguments can be 'audio' 'metadata' or/and 'tempos'
    """

    # use the default location: ~/mir_datasets/giantsteps_tempo
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_message = """
            Unfortunately the audio files of the Giant Steps Tempo dataset are not available
            for download. If you have the Giant Steps audio dataset, place the contents into
            a folder called GiantSteps_tempo with the following structure:
                > GiantSteps_tempo/
                    > giantsteps-tempo-dataset-0b7d47ba8cae59d3535a02e3db69e2cf6d0af5bb/
                    > audio/
            and copy the folder to {}
        """.format(
        data_home
    )

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        info_message=download_message,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )


def validate(data_home=None, silence=False):
    """Validate if a local version of this dataset is consistent
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths where the expected file exists locally
            but has a different checksum than the reference
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Get the list of track IDs for this dataset
    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load giantsteps_tempo dataset
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    giantsteps_key_data = {}
    for tempo in track_ids():
        giantsteps_key_data[tempo] = Track(tempo, data_home=data_home)
    return giantsteps_key_data


def load_genre(path):
    """Load genre data from a file
    Args:
        path (str): path to metadata annotation file
    Returns:
        (str): loaded genre data
    """
    if path is None:
        return None

    with open(path) as json_file:
        annotation = jams.load(json_file)

    return annotation.search(namespace='tag_open')[0]['data'][0].value


def load_tempo(tempo_path):
    """Load giantsteps_tempo tempo data from a file ordered by confidence
    Args:
        tempo_path (str): path to tempo annotation file
    Returns:
        (list of utils.TempoData): loaded tempo data
    """
    if tempo_path is None:
        return None

    if not os.path.exists(tempo_path):
        raise IOError("tempo_path {} does not exist".format(tempo_path))

    with open(tempo_path) as json_file:
        annotation = jams.load(json_file)

    tempo = annotation.search(namespace='tempo')[0]['data']

    return utils.TempoData(
        np.array([t.time for t in tempo]),
        np.array([t.duration for t in tempo]),
        np.array([t.value for t in tempo]),
        np.array([t.confidence for t in tempo]),
    )


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========
Peter Knees, Ángel Faraldo, Perfecto Herrera, Richard Vogl,
Sebastian Böck, Florian Hörschläger, Mickael Le Goff: "Two data
sets for tempo estimation and key detection in electronic dance
music annotated from user corrections," Proc. of the 16th
Conference of the International Society for Music Information
Retrieval (ISMIR'15), Oct. 2015, Malaga, Spain.
========== Bibtex ==========
@inproceedings{knees2015two,
  title={Two data sets for tempo estimation and key detection in electronic dance music annotated from user corrections},
  author={Knees, Peter and Faraldo P{\'e}rez, {\'A}ngel and Boyer, Herrera and Vogl, Richard and B{\"o}ck, Sebastian and H{\"o}rschl{\"a}ger, Florian and Le Goff, Mickael and others},
  booktitle={Proceedings of the 16th International Society for Music Information Retrieval Conference (ISMIR); 2015 Oct 26-30; M{\'a}laga, Spain.[M{\'a}laga]: International Society for Music Information Retrieval, 2015. p. 364-70.},
  year={2015},
  organization={International Society for Music Information Retrieval (ISMIR)}
}
===========  MLA ===========
Hendrik Schreiber, Meinard Müller: "A Crowdsourced Experiment
for Tempo Estimation of Electronic Dance Music", Proc. of the
19th Conference of the International Society for Music
Information Retrieval (ISMIR'18), Sept. 2018, Paris, France.
========== Bibtex ==========
@inproceedings{SchreiberM18a_Tempo_ISMIR,
author    = {Hendrik Schreiber and Meinard M{\"u}ller},
title     = {A Crowdsourced Experiment for Tempo Estimation of Electronic Dance Music},
booktitle = {Proceedings of the International Conference on Music Information Retrieval ({ISMIR})},
address   = {Paris, France},
year      = {2018},
url-pdf   = {http://www.tagtraum.com/download/2018_schreiber_tempo_giantsteps.pdf}
}




    """
    print(cite_data)



# -*- coding: utf-8 -*-
"""beatport_key Dataset Loader
The Beatport EDM Key Dataset includes 1486 two-minute sound excerpts from various EDM
subgenres, annotated with single-key labels, comments and confidence levels generously provided by Eduard Mas Marín,
and thoroughly revised and expanded by Ángel Faraldo.

The original audio samples belong to online audio snippets from Beatport, an online music store for DJ's and
Electronic Dance Music Producers (<http:\\www.beatport.com>). If this dataset were used in further research,
we would appreciate the citation of the current DOI (10.5281/zenodo.1101082) and the following doctoral dissertation,
where a detailed description of the properties of this dataset can be found:

Ángel Faraldo (2017). Tonality Estimation in Electronic Dance Music: A Computational and Musically Informed
Examination. PhD Thesis. Universitat Pompeu Fabra, Barcelona.

This dataset is mainly intended to assess the performance of computational key estimation algorithms in electronic
dance music subgenres.

Data License: Creative Commons Attribution Share Alike 4.0 International
"""
import fnmatch
import json
import librosa
import os

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils

DATASET_DIR = 'beatport_key'
REMOTES = {
    'keys': download_utils.RemoteFileMetadata(
        filename='keys.zip',
        url='https://zenodo.org/record/1101082/files/keys.zip?download=1',
        checksum='939abc05f36121badfac4087241ac172',
        destination_dir='.',
    ),
    'metadata': download_utils.RemoteFileMetadata(
        filename='original_metadata.zip',
        url='https://zenodo.org/record/1101082/files/original_metadata.zip?download=1',
        checksum='bb3e3ac1fe5dee7600ef2814accdf8f8',
        destination_dir='.',
    ),
    'audio': download_utils.RemoteFileMetadata(
        filename='audio.zip',
        url='https://zenodo.org/record/1101082/files/audio.zip?download=1',
        checksum='f490ee6c23578482d6fcfa11b82636a1',
        destination_dir='.',
    ),
}

DATA = utils.LargeData('beatport_key_index.json')


class Track(track.Track):
    """beatport_key track class
    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Attributes:
        audio_path (str): track audio path
        keys_path (str): key annotation path
        metadata_path (str): sections annotation path
        title (str): title of the track
        track_id (str): track id
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError(
                '{} is not a valid track ID in beatport_key'.format(track_id)
            )

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]
        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])
        self.keys_path = os.path.join(self._data_home, self._track_paths['key'][0])
        self.metadata_path = (
            os.path.join(self._data_home, self._track_paths['meta'][0])
            if self._track_paths['meta'][0] is not None
            else None
        )
        self.title = self.audio_path.replace(".mp3", '').split('/')[-1]

    @utils.cached_property
    def key(self):
        """String: key annotation"""
        return load_key(self.keys_path)

    @utils.cached_property
    def artists(self):
        """Dict: artist annotation"""
        return load_artist(self.metadata_path)

    @utils.cached_property
    def genres(self):
        """Dict: genre annotation"""
        return load_genre(self.metadata_path)

    @utils.cached_property
    def tempo(self):
        """int: tempo beatports crowdsourced annotation"""
        return load_tempo(self.metadata_path)

    @property
    def audio(self):
        """(np.ndarray, float): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path,
            metadata={
                'artists': self.artists,
                'genres': self.genres,
                'tempo': self.tempo,
                'title': self.title,
                'key': self.key,
            },
        )


def load_audio(audio_path):
    """Load a beatport_key audio file.
    Args:
        audio_path (str): path to audio file
    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file
    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))
    return librosa.load(audio_path, sr=None, mono=True)


def find_replace(directory, find, replace, pattern):
    """
    Replace in some directory all the songs with the format pattern find by replace
    Parameters
    ----------
    directory (str) path to directory
    find (str) string from replace
    replace (str) string to replace
    pattern (str) regex that must match the directories searrched
    """
    for path, dirs, files in os.walk(os.path.abspath(directory)):
        for filename in fnmatch.filter(files, pattern):
            filepath = os.path.join(path, filename)
            with open(filepath) as f:
                s = f.read()
            s = s.replace(find, replace)
            with open(filepath, "w") as f:
                f.write(s)


def download(
    data_home=None, force_overwrite=False, cleanup=True, partial_download=None
):
    """Download the beatport_key Dataset (annotations).
    The audio files are not provided due to copyright issues.

    This dataset annotations have characters that doesnt correspond with json format. In particular, "bpm": nan
    doesn't correspond to json format. The function find_replace is used to fix this problem.
    input file
    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.
        partial_download(list of str)
            arguments can be 'audio' 'metadata' or/and 'keys'
    """

    # use the default location: ~/mir_datasets/beatport_key
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_message = ""

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        partial_download=partial_download,
        info_message=download_message,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )
    # removing nans from JSON files
    find_replace(os.path.join(data_home, "meta"), ": nan", ": null", "*.json")


def validate(data_home=None, silence=False):
    """Validate if a local version of this dataset is consistent
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths where the expected file exists locally
            but has a different checksum than the reference
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Get the list of track IDs for this dataset
    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load beatport_key dataset
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    beatles_data = {}
    for key in track_ids():
        beatles_data[key] = Track(key, data_home=data_home)
    return beatles_data


def load_key(keys_path):
    """Load beatport_key format key data from a file
    Args:
        keys_path (str): path to key annotation file
    Returns:
        (str): loaded key data
    """
    if keys_path is None:
        return None

    if not os.path.exists(keys_path):
        raise IOError("keys_path {} does not exist".format(keys_path))

    with open(keys_path) as f:
        key = f.readline()

    return key


def load_tempo(metadata_path):
    """Load beatport_key tempo data from a file
    Args:
        metadata_path (str): path to metadata annotation file
    Returns:
        (str): loaded tempo data
    """
    if metadata_path is None:
        return None

    if not os.path.exists(metadata_path):
        raise IOError("metadata_path {} does not exist".format(metadata_path))

    with open(metadata_path) as json_file:
        meta = json.load(json_file)

    return meta["bpm"]


def load_genre(metadata_path):
    """Load beatport_key genre data from a file
    Args:
        metadata_path (str): path to metadata annotation file
    Returns:
        (dict): with the list of strings with genres ['genres'] and list of strings with sub-genres ['sub_genres']
    """
    if metadata_path is None:
        return None

    if not os.path.exists(metadata_path):
        raise IOError("metadata_path {} does not exist".format(metadata_path))

    with open(metadata_path) as json_file:
        meta = json.load(json_file)

    return {
        "genres": [genre["name"] for genre in meta["genres"]],
        "sub_genres": [genre["name"] for genre in meta["sub_genres"]],
    }


def load_artist(metadata_path):
    """Load beatport_key tempo data from a file
    Args:
        metadata_path (str): path to metadata annotation file
    Returns:
        (list of strings): list of artists involved in the track.
    """
    if metadata_path is None:
        return None

    if not os.path.exists(metadata_path):
        raise IOError("metadata_path {} does not exist".format(metadata_path))

    with open(metadata_path) as json_file:
        meta = json.load(json_file)

    return [artist["name"] for artist in meta["artists"]]


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========
Ángel Faraldo (2017).
Tonality Estimation in Electronic Dance Music: A Computational and Musically Informed Examination.
PhD Thesis. Universitat Pompeu Fabra, Barcelona.
========== Bibtex ==========
@phdthesis {3897,
    title = {Tonality Estimation in Electronic Dance Music: A Computational and Musically Informed Examination},
    year = {2018},
    month = {03/2018},
    pages = {234},
    school = {Universitat Pompeu Fabra},
    address = {Barcelona},
    abstract = {This dissertation revolves around the task of computational key estimation in electronic dance music, upon which three interrelated operations are performed. First, I attempt to detect possible misconceptions within the task, which is typically accomplished with a tonal vocabulary overly centred in Western classical tonality, reduced to a binary major/minor model which might not accomodate popular music styles. Second, I present a study of tonal practises in electronic dance music, developed hand in hand with the curation of a corpus of over 2,000 audio excerpts, including various subgenres and degrees of complexity. Based on this corpus, I propose the creation of more open-ended key labels, accounting for other modal practises and ambivalent tonal configurations. Last, I describe my own key finding methods, adapting existing models to the musical idiosyncrasies and tonal distributions of electronic dance music, with new statistical key profiles derived from the newly created corpus.},
    keywords = {EDM, Electronic Dance Music, Key Estimation, mir, music information retrieval, tonality},
    url = {https://doi.org/10.5281/zenodo.1154586},
    author = {{\'A}ngel Faraldo}
}

    """

    print(cite_data)



# -*- coding: utf-8 -*-
"""MAESTRO Dataset Loader

MAESTRO (MIDI and Audio Edited for Synchronous TRacks and Organization) is a
dataset composed of over 200 hours of virtuosic piano performances captured
with fine alignment (~3 ms) between note labels and audio waveforms.

The dataset is created and released by Google's Magenta team.

The dataset contains over 200 hours of paired audio and MIDI recordings from
ten years of International Piano-e-Competition. The MIDI data includes key
strike velocities and sustain/sostenuto/una corda pedal positions. Audio and
MIDI files are aligned with ∼3 ms accuracy and sliced to individual musical
pieces, which are annotated with composer, title, and year of performance.
Uncompressed audio is of CD quality or higher (44.1–48 kHz 16-bit PCM stereo).

A train/validation/test split configuration is also proposed, so that the same
composition, even if performed by multiple contestants, does not appear in
multiple subsets. Repertoire is mostly classical, including composers from the
17th to early 20th century.

The dataset is made available by Google LLC under a Creative Commons
Attribution Non-Commercial Share-Alike 4.0 (CC BY-NC-SA 4.0) license.

This loader supports MAESTRO version 2.

For more details, please visit: https://magenta.tensorflow.org/datasets/maestro
"""

import json
import glob
import logging
import os
import shutil

import librosa
import numpy as np
import pretty_midi

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils

DATASET_DIR = 'MAESTRO'

REMOTES = {
    'all': download_utils.RemoteFileMetadata(
        filename='maestro-v2.0.0.zip',
        url='https://storage.googleapis.com/magentadata/datasets/maestro/v2.0.0/maestro-v2.0.0.zip',
        checksum='7a6c23536ebcf3f50b1f00ac253886a7',
        destination_dir='',
    ),
    'midi': download_utils.RemoteFileMetadata(
        filename='maestro-v2.0.0-midi.zip',
        url='https://storage.googleapis.com/magentadata/datasets/maestro/v2.0.0/maestro-v2.0.0-midi.zip',
        checksum='8a45cc678a8b23cd7bad048b1e9034c5',
        destination_dir='',
    ),
    'metadata': download_utils.RemoteFileMetadata(
        filename='maestro-v2.0.0.json',
        url='https://storage.googleapis.com/magentadata/datasets/maestro/v2.0.0/maestro-v2.0.0.json',
        checksum='576172af1cdc4efddcf0be7d260d48f7',
        destination_dir='maestro-v2.0.0',
    ),
}


def _load_metadata(data_home):
    metadata_path = os.path.join(data_home, 'maestro-v2.0.0.json')
    if not os.path.exists(metadata_path):
        logging.info("Metadata file {} not found.".format(metadata_path))
        return None

    # load metadata however makes sense for your dataset
    with open(metadata_path, 'r') as fhandle:
        raw_metadata = json.load(fhandle)

    metadata = {}
    for mdata in raw_metadata:
        track_id = mdata['midi_filename'].split('.')[0]
        metadata[track_id] = mdata

    metadata['data_home'] = data_home

    return metadata


DATA = utils.LargeData('maestro_index.json', _load_metadata)


class Track(track.Track):
    """MAESTRO Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored. default=None
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        audio_path (str): Path to the track's audio file
        canonical_composer (str): Composer of the piece, standardized on a
            single spelling for a given name.
        canonical_title (str): Title of the piece. Not guaranteed to be
            standardized to a single representation.
        duration (float): Duration in seconds, based on the MIDI file.
        midi_path (str): Path to the track's MIDI file
        split (str): Suggested train/validation/test split.
        track_id (str): track id
        year (int): Year of performance.

    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in MAESTRO'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])
        self.midi_path = os.path.join(self._data_home, self._track_paths['midi'][0])

        self._metadata = DATA.metadata(data_home)
        if self._metadata is not None and track_id in self._metadata:
            self.canonical_composer = self._metadata[track_id]['canonical_composer']
            self.canonical_title = self._metadata[track_id]['canonical_title']
            self.split = self._metadata[track_id]['split']
            self.year = self._metadata[track_id]['year']
            self.duration = self._metadata[track_id]['duration']
        else:
            self.canonical_composer = None
            self.canonical_title = None
            self.split = None
            self.year = None
            self.duration = None

    @utils.cached_property
    def midi(self):
        """output type: description of output"""
        return load_midi(self.midi_path)

    @utils.cached_property
    def notes(self):
        """NoteData: annotated piano notes"""
        return load_notes(self.midi_path, self.midi)

    @property
    def audio(self):
        """(np.ndarray, float): track's audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path,
            note_data=[(self.notes, None)],
            metadata=self._metadata,
        )


def load_midi(midi_path):
    """Load a MAESTRO midi file.

    Args:
        midi_path (str): path to midi file

    Returns:
        midi_data (obj): pretty_midi object

    """
    if not os.path.exists(midi_path):
        raise IOError("midi_path {} does not exist".format(midi_path))

    return pretty_midi.PrettyMIDI(midi_path)


def load_notes(midi_path, midi=None):
    """Load note data from the midi file.

    Args:
        midi_path (str): path to midi file
        midi (pretty_midi.PrettyMIDI): pre-loaded midi object or None
            if None, the midi object is loaded using midi_path

    Returns:
        note_data (NoteData)

    """
    if midi is None:
        midi = load_midi(midi_path)

    intervals = []
    pitches = []
    confidence = []
    for note in midi.instruments[0].notes:
        intervals.append([note.start, note.end])
        pitches.append(librosa.midi_to_hz(note.pitch))
        confidence.append(note.velocity)
    return utils.NoteData(np.array(intervals), np.array(pitches), np.array(confidence))


def load_audio(audio_path):
    """Load a MAESTRO audio file.

    Args:
        audio_path (str): path to audio file

    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file

    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))
    return librosa.load(audio_path, sr=None, mono=True)


def download(
    data_home=None, partial_download=None, force_overwrite=False, cleanup=True
):
    """Download the dataset.

    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        partial_download (list):
            List indicating what to partially download. The list can include any of:
                * 'all': audio, midi and metadata
                * 'midi': midi and metadata only
                * 'metadata': metadata only
            If `None`, all data is downloaded.
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    # in MAESTRO "metadata" is contained in "midi" is contained in "all"
    if partial_download is None or 'all' in partial_download:
        partial_download = ['all']
    elif 'midi' in partial_download:
        partial_download = ['midi']

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        partial_download=partial_download,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )

    # files get downloaded to a folder called maestro-v2.0.0
    # move everything up a level
    maestro_dir = os.path.join(data_home, 'maestro-v2.0.0')
    maestro_files = glob.glob(os.path.join(maestro_dir, '*'))

    for fpath in maestro_files:
        shutil.move(fpath, data_home)

    if os.path.exists(maestro_dir):
        shutil.rmtree(maestro_dir)


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load MAESTRO dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    data = {}
    for key in DATA.index.keys():
        data[key] = Track(key, data_home=data_home)
    return data


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Curtis Hawthorne, Andriy Stasyuk, Adam Roberts, Ian Simon, Cheng-Zhi Anna Huang,
  Sander Dieleman, Erich Elsen, Jesse Engel, and Douglas Eck. "Enabling
  Factorized Piano Music Modeling and Generation with the MAESTRO Dataset."
  In International Conference on Learning Representations, 2019.
========== Bibtex ==========
@inproceedings{
  hawthorne2018enabling,
  title={Enabling Factorized Piano Music Modeling and Generation with the {MAESTRO} Dataset},
  author={Curtis Hawthorne and Andriy Stasyuk and Adam Roberts and Ian Simon and Cheng-Zhi Anna Huang and Sander Dieleman and Erich Elsen and Jesse Engel and Douglas Eck},
  booktitle={International Conference on Learning Representations},
  year={2019},
  url={https://openreview.net/forum?id=r1lYRjC9F7},
}
"""
    print(cite_data)



# -*- coding: utf-8 -*-

import os

from tests.test_utils import run_track_tests

from mirdata import mridangam_stroke
from tests.test_utils import DEFAULT_DATA_HOME


def test_track_default_data_home():
    # test data home None
    track_default = mridangam_stroke.Track("224030")
    assert track_default._data_home == os.path.join(
        DEFAULT_DATA_HOME, "Mridangam-Stroke"
    )


def test_track():
    default_trackid = "224030"
    data_home = 'tests/resources/mir_datasets/Mridangam-Stroke'
    track = mridangam_stroke.Track(default_trackid, data_home=data_home)
    expected_attributes = {
        'audio_path': "tests/resources/mir_datasets/Mridangam-Stroke/mridangam_stroke_1.5/"
        + "B/224030__akshaylaya__bheem-b-001.wav",
        'track_id': "224030",
        'stroke_name': 'bheem',
        'tonic': 'B'
    }

    run_track_tests(track, expected_attributes, {})

    audio, sr = track.audio
    assert sr == 44100
    assert audio.shape == (35841,)


def test_to_jams():
    default_trackid = "224030"
    data_home = 'tests/resources/mir_datasets/Mridangam-Stroke'
    track = mridangam_stroke.Track(default_trackid, data_home=data_home)
    jam = track.to_jams()

    # Validate Mridangam schema
    assert jam.validate()

    # Test the stroke parser
    parsed_stroke = jam.annotations["tag_open"][0].data[0].value
    assert parsed_stroke == "bheem"
    assert parsed_stroke in mridangam_stroke.STROKE_DICT, "Stroke {} not in stroke dictionary".format(parsed_stroke)

    # Test the tonic parser
    parsed_tonic = jam.sandbox.tonic
    assert parsed_tonic == "B"
    assert parsed_tonic in mridangam_stroke.TONIC_DICT, "Stroke {} not in stroke dictionary".format(parsed_tonic)



# -*- coding: utf-8 -*-
import pytest


def pytest_addoption(parser):
    parser.addoption(
        "--local", action="store_true", default=False, help="run local tests"
    )
    parser.addoption("--dataset", type=str, default="", help="dataset to test locally")
    parser.addoption(
        "--skip-download", action="store_true", default=False, help="skip download step"
    )


# @pytest.fixture(scope='session')
# def local(request):
#     return request.config.getoption('--local')


@pytest.fixture(scope='session')
def skip_local(request):
    if request.config.getoption('--local'):
        pytest.skip()


@pytest.fixture(scope='session')
def skip_remote(request):
    if not request.config.getoption('--local'):
        pytest.skip()


@pytest.fixture(scope='session')
def test_dataset(request):
    return request.config.getoption('--dataset')


@pytest.fixture(scope='session')
def skip_download(request):
    return request.config.getoption('--skip-download')



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import orchset, utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'Beethoven-S3-I-ex1'
    data_home = 'tests/resources/mir_datasets/Orchset'
    track = orchset.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'Beethoven-S3-I-ex1',
        'audio_path_mono': 'tests/resources/mir_datasets/Orchset/'
        + 'audio/mono/Beethoven-S3-I-ex1.wav',
        'audio_path_stereo': 'tests/resources/mir_datasets/Orchset/'
        + 'audio/stereo/Beethoven-S3-I-ex1.wav',
        'melody_path': 'tests/resources/mir_datasets/Orchset/'
        + 'GT/Beethoven-S3-I-ex1.mel',
        'composer': 'Beethoven',
        'work': 'S3-I',
        'excerpt': '1',
        'predominant_melodic_instruments': ['strings', 'winds'],
        'alternating_melody': True,
        'contains_winds': True,
        'contains_strings': True,
        'contains_brass': False,
        'only_strings': False,
        'only_winds': False,
        'only_brass': False,
    }

    expected_property_types = {'melody': utils.F0Data}

    run_track_tests(track, expected_attributes, expected_property_types)

    y_mono, sr_mono = track.audio_mono
    assert sr_mono == 44100
    assert y_mono.shape == (44100 * 2,)

    y_stereo, sr_stereo = track.audio_stereo
    assert sr_stereo == 44100
    assert y_stereo.shape == (2, 44100 * 2)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/Orchset'
    track = orchset.Track('Beethoven-S3-I-ex1', data_home=data_home)
    jam = track.to_jams()

    f0s = jam.search(namespace='pitch_contour')[0]['data']
    assert [f0.time for f0 in f0s] == [0.0, 0.08, 0.09]
    assert [f0.duration for f0 in f0s] == [0.0, 0.0, 0.0]
    assert [f0.value for f0 in f0s] == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 622.254, 'index': 0, 'voiced': True},
    ]
    assert [f0.confidence for f0 in f0s] == [0.0, 0.0, 1.0]

    assert jam['sandbox']['alternating_melody'] == True


def test_load_melody():
    # load a file which exists
    melody_path = 'tests/resources/mir_datasets/Orchset/GT/Beethoven-S3-I-ex1.mel'
    melody_data = orchset.load_melody(melody_path)

    # check types
    assert type(melody_data) == utils.F0Data
    assert type(melody_data.times) is np.ndarray
    assert type(melody_data.frequencies) is np.ndarray
    assert type(melody_data.confidence) is np.ndarray

    # check values
    assert np.array_equal(melody_data.times, np.array([0.0, 0.08, 0.09]))
    assert np.array_equal(melody_data.frequencies, np.array([0.0, 0.0, 622.254]))
    assert np.array_equal(melody_data.confidence, np.array([0.0, 0.0, 1.0]))


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/Orchset'
    metadata = orchset._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['Beethoven-S3-I-ex1'] == {
        'predominant_melodic_instruments-raw': 'strings+winds',
        'predominant_melodic_instruments-normalized': ['strings', 'winds'],
        'alternating_melody': True,
        'contains_winds': True,
        'contains_strings': True,
        'contains_brass': False,
        'only_strings': False,
        'only_winds': False,
        'only_brass': False,
        'composer': 'Beethoven',
        'work': 'S3-I',
        'excerpt': '1',
    }
    assert metadata['Haydn-S94-Menuet-ex1'] == {
        'predominant_melodic_instruments-raw': 'string+winds',
        'predominant_melodic_instruments-normalized': ['strings', 'winds'],
        'alternating_melody': True,
        'contains_winds': True,
        'contains_strings': True,
        'contains_brass': False,
        'only_strings': False,
        'only_winds': False,
        'only_brass': False,
        'composer': 'Haydn',
        'work': 'S94-Menuet',
        'excerpt': '1',
    }
    assert metadata['Musorgski-Ravel-PicturesExhibition-Promenade1-ex2'] == {
        'predominant_melodic_instruments-raw': 'strings',
        'predominant_melodic_instruments-normalized': ['strings'],
        'alternating_melody': False,
        'contains_winds': True,
        'contains_strings': False,
        'contains_brass': False,
        'only_strings': True,
        'only_winds': False,
        'only_brass': False,
        'composer': 'Musorgski-Ravel',
        'work': 'PicturesExhibition-Promenade1',
        'excerpt': '2',
    }
    assert metadata['Rimski-Korsakov-Scheherazade-YoungPrincePrincess-ex4'] == {
        'predominant_melodic_instruments-raw': 'strings+winds',
        'predominant_melodic_instruments-normalized': ['strings', 'winds'],
        'alternating_melody': True,
        'contains_winds': True,
        'contains_strings': True,
        'contains_brass': False,
        'only_strings': False,
        'only_winds': False,
        'only_brass': False,
        'composer': 'Rimski-Korsakov',
        'work': 'Scheherazade-YoungPrincePrincess',
        'excerpt': '4',
    }
    assert metadata['Schubert-S8-II-ex2'] == {
        'predominant_melodic_instruments-raw': 'winds (solo)',
        'predominant_melodic_instruments-normalized': ['winds'],
        'alternating_melody': False,
        'contains_winds': False,
        'contains_strings': True,
        'contains_brass': False,
        'only_strings': False,
        'only_winds': True,
        'only_brass': False,
        'composer': 'Schubert',
        'work': 'S8-II',
        'excerpt': '2',
    }

    metadata_none = orchset._load_metadata('asdf/asdf')
    assert metadata_none is None



# -*- coding: utf-8 -*-
"""RWC Jazz Dataset Loader.

The Jazz Music Database consists of 50 pieces:

* Instrumentation variations: 35 pieces (5 pieces × 7 instrumentations).
The instrumentation-variation pieces were recorded to obtain different versions
of the same piece; i.e., different arrangements performed by different player
instrumentations. Five standard-style jazz pieces were originally composed
and then performed in modern-jazz style using the following seven instrumentations:
1. Piano solo
2. Guitar solo
3. Duo: Vibraphone + Piano, Flute + Piano, and Piano + Bass
4. Piano trio: Piano + Bass + Drums
5. Piano trio + Trumpet or Tenor saxophone
6. Octet: Piano trio + Guitar + Alto saxophone + Baritone saxophone + Tenor saxophone × 2
7. Piano trio + Vibraphone or Flute

* Style variations: 9 pieces
The style-variation pieces were recorded to represent various styles of jazz.
They include four well-known public-domain pieces and consist of
1. Vocal jazz: 2 pieces (including "Aura Lee")
2. Big band jazz: 2 pieces (including "The Entertainer")
3. Modal jazz: 2 pieces
4. Funky jazz: 2 pieces (including "Silent Night")
5. Free jazz: 1 piece (including "Joyful, Joyful, We Adore Thee")
Fusion (crossover): 6 pieces
The fusion pieces were recorded to obtain music that combines elements of jazz
with other styles such as popular, rock, and latin. They include music with an
eighth-note feel, music with a sixteenth-note feel, and Latin jazz music.

For more details, please visit: https://staff.aist.go.jp/m.goto/RWC-MDB/rwc-mdb-j.html
"""
import csv
import librosa
import logging
import os

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils

# these functions are identical for all rwc datasets
from mirdata.rwc_classical import (
    load_beats,
    load_sections,
    load_audio,
    _duration_to_sec,
)


REMOTES = {
    'metadata': download_utils.RemoteFileMetadata(
        filename='rwc-j.csv',
        url='https://github.com/magdalenafuentes/metadata/archive/master.zip',
        checksum='7dbe87fedbaaa1f348625a2af1d78030',
        destination_dir=None,
    ),
    'annotations_beat': download_utils.RemoteFileMetadata(
        filename='AIST.RWC-MDB-J-2001.BEAT.zip',
        url='https://staff.aist.go.jp/m.goto/RWC-MDB/AIST-Annotation/AIST.RWC-MDB-J-2001.BEAT.zip',
        checksum='b483853da05d0fff3992879f7729bcb4',
        destination_dir='annotations',
    ),
    'annotations_sections': download_utils.RemoteFileMetadata(
        filename='AIST.RWC-MDB-J-2001.CHORUS.zip',
        url='https://staff.aist.go.jp/m.goto/RWC-MDB/AIST-Annotation/AIST.RWC-MDB-J-2001.CHORUS.zip',
        checksum='44afcf7f193d7e48a7d99e7a6f3ed39d',
        destination_dir='annotations',
    ),
}

DATASET_DIR = 'RWC-Jazz'


def _load_metadata(data_home):

    metadata_path = os.path.join(data_home, 'metadata-master', 'rwc-j.csv')

    if not os.path.exists(metadata_path):
        logging.info(
            'Metadata file {} not found.'.format(metadata_path)
            + 'You can download the metadata file by running download()'
        )
        return None

    with open(metadata_path, 'r') as fhandle:
        dialect = csv.Sniffer().sniff(fhandle.read(1024))
        fhandle.seek(0)
        reader = csv.reader(fhandle, dialect)
        raw_data = []
        for line in reader:
            if line[0] != 'Piece No.':
                raw_data.append(line)

    metadata_index = {}
    for line in raw_data:
        if line[0] == 'Piece No.':
            continue
        p = '00' + line[0].split('.')[1][1:]
        track_id = 'RM-J{}'.format(p[len(p) - 3 :])

        metadata_index[track_id] = {
            'piece_number': line[0],
            'suffix': line[1],
            'track_number': line[2],
            'title': line[3],
            'artist': line[4],
            'duration': _duration_to_sec(line[5]),
            'variation': line[6],
            'instruments': line[7],
        }

    metadata_index['data_home'] = data_home

    return metadata_index


DATA = utils.LargeData('rwc_jazz_index.json', _load_metadata)


class Track(track.Track):
    """rwc_jazz Track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored. default=None
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        artist (str): Artist name
        audio_path (str): path of the audio file
        beats_path (str): path of the beat annotation file
        duration (float): Duration of the track in seconds
        instruments (str): list of used instruments.
        piece_number (str): Piece number of this Track, [1-50]
        sections_path (str): path of the section annotation file
        suffix (str): M01-M04
        title (str): Title of The track.
        track_id (str): track id
        track_number (str): CD track number of this Track
        variation (str): TODO

    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in RWC-Jazz'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)
        self._data_home = data_home

        self._track_paths = DATA.index[track_id]
        self.sections_path = os.path.join(
            self._data_home, self._track_paths['sections'][0]
        )
        self.beats_path = os.path.join(self._data_home, self._track_paths['beats'][0])

        metadata = DATA.metadata(data_home)
        if metadata is not None and track_id in metadata:
            self._track_metadata = metadata[track_id]
        else:
            self._track_metadata = {
                'piece_number': None,
                'suffix': None,
                'track_number': None,
                'title': None,
                'artist': None,
                'duration': None,
                'variation': None,
                'instruments': None,
            }

        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])

        self.piece_number = self._track_metadata['piece_number']
        self.suffix = self._track_metadata['suffix']
        self.track_number = self._track_metadata['track_number']
        self.title = self._track_metadata['title']
        self.artist = self._track_metadata['artist']
        self.duration = self._track_metadata['duration']
        self.variation = self._track_metadata['variation']
        self.instruments = self._track_metadata['instruments']

    @utils.cached_property
    def sections(self):
        """SectionData: human-labeled section data"""
        return load_sections(self.sections_path)

    @utils.cached_property
    def beats(self):
        """BeatData: human-labeled beat data"""
        return load_beats(self.beats_path)

    @property
    def audio(self):
        """(np.ndarray, float): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path,
            beat_data=[(self.beats, None)],
            section_data=[(self.sections, None)],
            metadata=self._track_metadata,
        )


def download(
    data_home=None, partial_download=None, force_overwrite=False, cleanup=True
):
    """Download the RWC Jazz (annotations and metadata).
    The audio files are not provided due to copyright issues.

    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        partial_download (list):
             List indicating what to partially download. The list can include any of:
             * `'annotations_beat'` the beat annotation files
             * `'annotations_sections'` the sections annotation files
             * `'metadata'` the metadata files
             If `None`, all data is downloaded.
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    info_message = """
        Unfortunately the audio files of the RWC-Jazz dataset are not available
        for download. If you have the RWC-Jazz dataset, place the contents into a
        folder called RWC-Jazz with the following structure:
            > RWC-Jazz/
                > annotations/
                > audio/rwc-j-m0i with i in [1 .. 4]
                > metadata-master/
        and copy the RWC-Jazz folder to {}
    """.format(
        data_home
    )

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        partial_download=partial_download,
        info_message=info_message,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load RWC-Jazz dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    rwc_jazz_data = {}
    for key in track_ids():
        rwc_jazz_data[key] = Track(key, data_home=data_home)
    return rwc_jazz_data


def cite():
    cite_data = """
===========  MLA ===========

Goto, Masataka, et al.,
"RWC Music Database: Popular, Classical and Jazz Music Databases.",
3rd International Society for Music Information Retrieval Conference (2002)

========== Bibtex ==========

@inproceedings{goto2002rwc,
  title={RWC Music Database: Popular, Classical and Jazz Music Databases.},
  author={Goto, Masataka and Hashiguchi, Hiroki and Nishimura, Takuichi and Oka, Ryuichi},
  booktitle={3rd International Society for Music Information Retrieval Conference},
  year={2002},
  series={ISMIR},
}

"""

    print(cite_data)



# -*- coding: utf-8 -*-
import os
import pretty_midi
import shutil

from mirdata import groove_midi, utils, download_utils
from tests.test_utils import run_track_tests


def test_track():
    default_trackid = 'drummer1/eval_session/1'
    data_home = 'tests/resources/mir_datasets/Groove-MIDI'
    track = groove_midi.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'drummer': 'drummer1',
        'session': 'drummer1/eval_session',
        'track_id': 'drummer1/eval_session/1',
        'style': 'funk/groove1',
        'tempo': 138,
        'beat_type': 'beat',
        'time_signature': '4-4',
        'midi_filename': 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.mid',
        'audio_filename': 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.wav',
        'midi_path': os.path.join(
            data_home, 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.mid'
        ),
        'audio_path': os.path.join(
            data_home, 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.wav'
        ),
        'duration': 27.872308,
        'split': 'test',
    }

    expected_property_types = {
        'beats': utils.BeatData,
        'drum_events': utils.EventData,
        'midi': pretty_midi.PrettyMIDI,
    }

    assert track._track_paths == {
        'audio': [
            'drummer1/eval_session/1_funk-groove1_138_beat_4-4.wav',
            '7f94a191506f70ac9d313b7978203c3c',
        ],
        'midi': [
            'drummer1/eval_session/1_funk-groove1_138_beat_4-4.mid',
            'b01a609cee84cfbc2c154bb9b6566955',
        ],
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    audio, sr = track.audio
    assert sr == 22050
    assert audio.shape == (613566,)

    # test midi loading functions
    midi_data = track.midi
    assert len(midi_data.instruments) == 1
    assert len(midi_data.instruments[0].notes) == 410
    assert midi_data.estimate_tempo() == 198.7695135305443
    assert midi_data.get_piano_roll().shape == (128, 2787)


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/Groove-MIDI'
    metadata = groove_midi._load_metadata(data_home)

    assert metadata['data_home'] == data_home
    assert metadata['drummer1/eval_session/1'] == {
        'drummer': 'drummer1',
        'session': 'drummer1/eval_session',
        'track_id': 'drummer1/eval_session/1',
        'style': 'funk/groove1',
        'tempo': 138,
        'beat_type': 'beat',
        'time_signature': '4-4',
        'midi_filename': 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.mid',
        'audio_filename': 'drummer1/eval_session/1_funk-groove1_138_beat_4-4.wav',
        'duration': 27.872308,
        'split': 'test',
    }
    metadata_none = groove_midi._load_metadata('asdf/asdf')
    assert metadata_none is None


def test_load_audio():
    audio_none, sr_none = groove_midi.load_audio(None)
    assert audio_none is None
    assert sr_none is None


def test_download(httpserver):
    data_home = 'tests/resources/mir_datasets/Groove-MIDI_download'
    if os.path.exists(data_home):
        shutil.rmtree(data_home)

    httpserver.serve_content(
        open('tests/resources/download/groove-v1-0.0.zip', 'rb').read()
    )

    groove_midi.REMOTES = {
        'all': download_utils.RemoteFileMetadata(
            filename='groove-v1-0.0.zip',
            url=httpserver.url,
            checksum=('97a9a888d2a65cc87bb26e74df08b011'),
            destination_dir=None,
        )
    }
    groove_midi.download(data_home=data_home)

    assert os.path.exists(data_home)
    assert not os.path.exists(os.path.join(data_home, 'groove'))

    assert os.path.exists(os.path.join(data_home, "info.csv"))
    track = groove_midi.Track('drummer1/eval_session/1', data_home=data_home)
    assert os.path.exists(track.midi_path)
    assert os.path.exists(track.audio_path)

    if os.path.exists(data_home):
        shutil.rmtree(data_home)



# -*- coding: utf-8 -*-

"""
Mridangam Stroke Dataset Loader

The Mridangam Stroke dataset is a collection of individual strokes of
the Mridangam in various tonics. The dataset comprises of 10 different
strokes played on Mridangams with 6 different tonic values. The audio
examples were recorded from a professional Carnatic percussionist in a
semi-anechoic studio conditions by Akshay Anantapadmanabhan.

Total audio samples: 6977

Used microphones:
* SM-58 microphones
* H4n ZOOM recorder.

Audio specifications
* Sampling frequency: 44.1 kHz
* Bit-depth: 16 bit
* Audio format: .wav

The dataset can be used for training models for each Mridangam stroke. The
presentation of the dataset took place on the IEEE International Conference
on Acoustics, Speech and Signal Processing (ICASSP 2013) on May 2013.
You can read the full publication here: https://repositori.upf.edu/handle/10230/25756

Mridangam Dataset is annotated by storing the informat of each track in their filenames.
The structure of the filename is:
<TrackID>__<AuthorName>__<StrokeName>-<Tonic>-<InstanceNum>.wav

The dataset is made available by CompMusic under a Creative Commons
Attribution 3.0 Unported (CC BY 3.0) License.

For more details, please visit: https://compmusic.upf.edu/mridangam-stroke-dataset
"""

import os
import librosa

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils


DATASET_DIR = 'Mridangam-Stroke'


REMOTES = {
    'remote_data': download_utils.RemoteFileMetadata(
        filename='mridangam_stroke_1.5.zip',
        url='https://zenodo.org/record/4068196/files/mridangam_stroke_1.5.zip?download=1',
        checksum='39af55b2476b94c7946bec24331ec01a',  # the md5 checksum
        destination_dir=None,  # relative path for where to unzip the data, or None
    ),
}

DATA = utils.LargeData(
    'mridangam_stroke_index.json'
)


STROKE_DICT = {
    'bheem', 'cha', 'dheem', 'dhin', 'num', 'ta', 'tha', 'tham', 'thi', 'thom'
}


TONIC_DICT = {
    'B', 'C', 'C#', 'D', 'D#', 'E'
}


class Track(track.Track):
    """Mridangam Stroke track class
    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets/Mridangam-Stroke`
    Attributes:
        track_id (str): track id
        audio_path (str): audio path
        stroke_name (str): name of the Mridangam stroke present in Track
        tonic (str): tonic of the stroke in the Track
    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in Example'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]

        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])

        # Parse stroke name annotation from audio file name
        self.stroke_name = self.audio_path.split('__')[2].split('-')[0]
        assert self.stroke_name in STROKE_DICT, "Stroke {} not in stroke dictionary".format(self.stroke_name)

        # Parse tonic annotation from audio file name
        self.tonic = os.path.basename(os.path.dirname(self.audio_path))
        assert self.tonic in TONIC_DICT, "Tonic {} not in tonic dictionary".format(self.tonic)

    @property
    def audio(self):
        """(String): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path,
            tags_open_data=[(self.stroke_name, 'stroke_name')],
            metadata={
                'tonic': self.tonic
            },
        )


def load_audio(audio_path):
    """Load a Mridangam Stroke Dataset audio file.
    Args:
        audio_path (str): path to audio file
    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file
    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))
    return librosa.load(audio_path, sr=44100, mono=True)


def download(data_home=None, force_overwrite=False, cleanup=True):
    """Download the Mridangam Stroke Dataset.
    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        info_message=None,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )


def validate(data_home=None, silence=False):
    """Validate if the stored dataset is a valid version
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths that file exists in the dataset
            index but has a different checksum compare to the reference checksum
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Return track ids
    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load Mridangam Stroke dataset
    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
    Returns:
        (dict): {`track_id`: track data}
    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    data = {}
    for key in DATA.index.keys():
        data[key] = Track(key, data_home=data_home)
    return data


def cite():
    """Print the reference"""

    cite_data = """
=========== MLA ===========
Anantapadmanabhan, A., Bellur, A., & Murthy, H. A. 
"Modal analysis and transcription of strokes of the mridangam using non-negative matrix factorization" (2013)
IEEE International Conference on Acoustics, Speech and Signal Processing (ICASSP 2013)
========== Bibtex ==========
@article{Anantapadmanabhan2013,
    author = {Anantapadmanabhan, Akshay and Bellur, Ashwin and Murthy, Hema A.},
    doi = {10.1109/ICASSP.2013.6637633},
    isbn = {9781479903566},
    issn = {15206149},
    journal = {ICASSP, IEEE International Conference on Acoustics, Speech and Signal Processing - Proceedings},
    keywords = {Hidden Markov models, Modal Analysis, Mridangam, Non-negative Matrix Factorization,
    automatic transcription},
    pages = {181--185},
    title = {{Modal analysis and transcription of strokes of the mridangam using non-negative matrix factorization}},
    year = {2013}
}
"""
    print(cite_data)



# -*- coding: utf-8 -*-

import numpy as np

from mirdata import rwc_popular, utils
from tests.test_utils import run_track_tests


def test_track():

    default_trackid = 'RM-P001'
    data_home = 'tests/resources/mir_datasets/RWC-Popular'
    track = rwc_popular.Track(default_trackid, data_home=data_home)

    expected_attributes = {
        'track_id': 'RM-P001',
        'audio_path': 'tests/resources/mir_datasets/RWC-Popular/'
        + 'audio/rwc-p-m01/1.wav',
        'sections_path': 'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.CHORUS/RM-P001.CHORUS.TXT',
        'beats_path': 'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.BEAT/RM-P001.BEAT.TXT',
        'chords_path': 'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.CHORD/RWC_Pop_Chords/N001-M01-T01.lab',
        'voca_inst_path': 'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.VOCA_INST/RM-P001.VOCA_INST.TXT',
        'piece_number': 'No. 1',
        'suffix': 'M01',
        'track_number': 'Tr. 01',
        'title': 'Eien no replica',
        'artist': 'Kazuo Nishi',
        'singer_information': 'Male',
        'duration': 209,
        'tempo': '135',
        'instruments': 'Gt',
        'drum_information': 'Drum sequences',
    }

    expected_property_types = {
        'beats': utils.BeatData,
        'sections': utils.SectionData,
        'chords': utils.ChordData,
        'vocal_instrument_activity': utils.EventData,
    }

    run_track_tests(track, expected_attributes, expected_property_types)

    # test audio loading functions
    y, sr = track.audio
    assert sr == 44100
    assert y.shape == (44100 * 2,)


def test_to_jams():

    data_home = 'tests/resources/mir_datasets/RWC-Popular'
    track = rwc_popular.Track('RM-P001', data_home=data_home)
    jam = track.to_jams()

    beats = jam.search(namespace='beat')[0]['data']
    assert [beat.time for beat in beats] == [
        0.04,
        0.49,
        0.93,
        1.37,
        1.82,
        2.26,
        2.71,
        3.15,
    ]
    assert [beat.duration for beat in beats] == [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0]
    assert [beat.value for beat in beats] == [1, 2, 3, 4, 1, 2, 3, 4]
    assert [beat.confidence for beat in beats] == [
        None,
        None,
        None,
        None,
        None,
        None,
        None,
        None,
    ]

    segments = jam.search(namespace='segment')[0]['data']
    assert [segment.time for segment in segments] == [0.04, 10.26, 188.48, 202.71]
    assert [segment.duration for segment in segments] == [
        10.22,
        12.889999999999999,
        14.230000000000018,
        4.449999999999989,
    ]
    assert [segment.value for segment in segments] == [
        'intro',
        'chorus A',
        'bridge A',
        'ending',
    ]
    assert [segment.confidence for segment in segments] == [None, None, None, None]

    chords = jam.search(namespace='chord')[0]['data']
    assert [chord.time for chord in chords] == [0.0, 0.104, 3.646, 43.992, 44.494]
    assert [chord.duration for chord in chords] == [
        0.104,
        1.754,
        1.7409999999999997,
        0.5020000000000024,
        3.142000000000003,
    ]
    assert [chord.value for chord in chords] == [
        'N',
        'Ab:min',
        'E:maj',
        'Bb:maj(*3)',
        'C:min7',
    ]
    assert [chord.confidence for chord in chords] == [None, None, None, None, None]

    assert jam['file_metadata']['title'] == 'Eien no replica'
    assert jam['file_metadata']['artist'] == 'Kazuo Nishi'


def test_load_chords():
    chords_path = (
        'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.CHORD/RWC_Pop_Chords/N001-M01-T01.lab'
    )
    chord_data = rwc_popular.load_chords(chords_path)

    # check types
    assert type(chord_data) is utils.ChordData
    assert type(chord_data.intervals) is np.ndarray
    assert type(chord_data.labels) is list

    # check values
    assert np.array_equal(
        chord_data.intervals[:, 0], np.array([0.000, 0.104, 3.646, 43.992, 44.494])
    )
    assert np.array_equal(
        chord_data.intervals[:, 1], np.array([0.104, 1.858, 5.387, 44.494, 47.636])
    )
    assert np.array_equal(
        chord_data.labels, ['N', 'Ab:min', 'E:maj', 'Bb:maj(*3)', 'C:min7']
    )


def test_load_voca_inst():
    vocinst_path = (
        'tests/resources/mir_datasets/RWC-Popular/'
        + 'annotations/AIST.RWC-MDB-P-2001.VOCA_INST/RM-P001.VOCA_INST.TXT'
    )
    vocinst_data = rwc_popular.load_voca_inst(vocinst_path)

    # check types
    assert type(vocinst_data) is utils.EventData
    assert type(vocinst_data.start_times) is np.ndarray
    assert type(vocinst_data.end_times) is np.ndarray
    assert type(vocinst_data.event) is np.ndarray

    # check values
    assert np.array_equal(
        vocinst_data.start_times,
        np.array(
            [
                0.000,
                10.293061224,
                11.883492063,
                12.087845804,
                13.587460317,
                13.819387755,
                20.668707482,
                20.832653061,
            ]
        ),
    )
    assert np.array_equal(
        vocinst_data.end_times,
        np.array(
            [
                10.293061224,
                11.883492063,
                12.087845804,
                13.587460317,
                13.819387755,
                20.668707482,
                20.832653061,
                26.465306122,
            ]
        ),
    )
    assert np.array_equal(
        vocinst_data.event,
        np.array(
            ['b', 'm:withm', 'b', 'm:withm', 'b', 'm:withm', 'b', 's:electricguitar']
        ),
    )


def test_load_metadata():
    data_home = 'tests/resources/mir_datasets/RWC-Popular'
    metadata = rwc_popular._load_metadata(data_home)
    assert metadata['data_home'] == data_home
    assert metadata['RM-P001'] == {
        'piece_number': 'No. 1',
        'suffix': 'M01',
        'track_number': 'Tr. 01',
        'title': 'Eien no replica',
        'artist': 'Kazuo Nishi',
        'singer_information': 'Male',
        'duration': 209,
        'tempo': '135',
        'instruments': 'Gt',
        'drum_information': 'Drum sequences',
    }



# -*- coding: utf-8 -*-
"""functions for converting mirdata annotations to jams format
"""

import jams
import librosa
import os

from mirdata import utils


def jams_converter(
    audio_path=None,
    beat_data=None,
    chord_data=None,
    note_data=None,
    f0_data=None,
    section_data=None,
    multi_section_data=None,
    tempo_data=None,
    event_data=None,
    key_data=None,
    lyrics_data=None,
    tags_gtzan_data=None,
    tags_open_data=None,
    metadata=None,
):
    """Convert annotations from a track to JAMS format.

    Parameters
    ----------
    audio_path (str or None):
        A path to the corresponding audio file, or None. If provided,
        the audio file will be read to compute the duration. If None,
        'duration' must be a field in the metadata dictionary, or the
        resulting jam object will not validate.
    beat_data (list or None):
        A list of tuples of (BeatData, str), where str describes the annotation (e.g. 'beats_1').
    chord_data (list or None):
        A list of tuples of (ChordData, str), where str describes the annotation.
    note_data (list or None):
        A list of tuples of (NoteData, str), where str describes the annotation.
    f0_data (list or None):
        A list of tuples of (F0Data, str), where str describes the annotation.
    section_data (list or None):
        A list of tuples of (SectionData, str), where str describes the annotation.
    multi_section_data (list or None):
        A list of tuples. Tuples in multi_section_data should contain another
        list of tuples, indicating annotations in the different levels
        e.g. ([(segments0, level0), '(segments1, level1)], annotator) and a str
        indicating the annotator
    tempo_data (list or None):
        A list of tuples of (float, str), where float gives the tempo in bpm
        and str describes the annotation.
    event_data (list or None):
        A list of tuples of (EventData, str), where str describes the annotation.
    key_data (list or None):
        A list of tuples of (KeyData, str), where str describes the annotation.
    lyrics_data (list or None):
        A list of tuples of (LyricData, str), where str describes the annotation.
    tags_gtzan_data (list or None):
        A list of tuples of (str, str), where the first srt is the tag and the second
        is a descriptor of the annotation.
    tags_open_data (list or None):
        A list of tuples of (str, str), where the first srt is the tag and the second
        is a descriptor of the annotation.
    metadata (dict or None):
        A dictionary containing the track metadata.

    Returns
    -------
    jam: JAM object
        A JAM object with all the annotations.
    """

    jam = jams.JAMS()

    # duration
    duration = None
    if audio_path is not None:
        if os.path.exists(audio_path):
            duration = librosa.get_duration(filename=audio_path)
        else:
            raise OSError(
                'jams conversion failed because the audio file '
                + 'for this track cannot be found, and it is required'
                + 'to compute duration.'
            )

    # metadata
    if metadata is not None:
        for key in metadata:
            if (
                key == 'duration'
                and duration is not None
                and metadata[key] != duration
            ):
                print(
                    'Warning: duration provided in metadata does not'
                    + 'match the duration computed from the audio file.'
                    + 'Using the duration provided by the metadata.'
                )

            if metadata[key] is None:
                continue

            if hasattr(jam.file_metadata, key):
                setattr(jam.file_metadata, key, metadata[key])
            else:
                setattr(jam.sandbox, key, metadata[key])

    if jam.file_metadata.duration is None:
        jam.file_metadata.duration = duration

    # beats
    if beat_data is not None:
        if not isinstance(beat_data, list):
            raise TypeError('beat_data should be a list of tuples')
        for beats in beat_data:
            if not isinstance(beats, tuple):
                raise TypeError(
                    'beat_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(beats))
                )
            jam.annotations.append(beats_to_jams(beats))

    # sections
    if section_data is not None:
        if not isinstance(section_data, list):
            raise TypeError('section_data should be a list of tuples')
        for sections in section_data:
            if not isinstance(sections, tuple):
                raise TypeError(
                    'section_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(sections))
                )
            jam.annotations.append(sections_to_jams(sections))

    # multi-sections (sections with multiple levels)
    if multi_section_data is not None:
        if not isinstance(multi_section_data, list):
            raise TypeError('multi_section_data should be a list of tuples')
        for sections in multi_section_data:
            if not isinstance(sections, tuple):
                raise TypeError(
                    'multi_section_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(sections))
                )
            if not (
                isinstance(sections[0], list) and isinstance(sections[0][0], tuple)
            ):
                raise TypeError(
                    'tuples in multi_section_data should contain a '
                    + 'list of tuples, indicating annotations in the different '
                    + 'levels, e.g. ([(segments0, level0), '
                    + '(segments1, level1)], annotator)'
                )
            jam.annotations.append(multi_sections_to_jams(sections))

    # tempo
    if tempo_data is not None:
        if type(tempo_data) != list:
            raise TypeError('tempo_data should be a list of tuples')
        for tempo in tempo_data:
            if type(tempo) != tuple:
                raise TypeError(
                    'tempo_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(tempo))
                )
            jam.annotations.append(tempos_to_jams(tempo))

    # events
    if event_data is not None:
        if type(event_data) != list:
            raise TypeError('event_data should be a list of tuples')
        for events in event_data:
            if type(events) != tuple:
                raise TypeError(
                    'event_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(events))
                )
            jam.annotations.append(events_to_jams(events))

    # chords
    if chord_data is not None:
        if not isinstance(chord_data, list):
            raise TypeError('chord_data should be a list of tuples')
        for chords in chord_data:
            if not isinstance(chords, tuple):
                raise TypeError(
                    'chord_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(chords))
                )
            jam.annotations.append(chords_to_jams(chords))

    # notes
    if note_data is not None:
        if not isinstance(note_data, list):
            raise TypeError('note_data should be a list of tuples')
        for notes in note_data:
            if not isinstance(notes, tuple):
                raise TypeError(
                    'note_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(notes))
                )
            jam.annotations.append(notes_to_jams(notes))

    # keys
    if key_data is not None:
        if not isinstance(key_data, list):
            raise TypeError('key_data should be a list of tuples')
        for keys in key_data:
            if not isinstance(keys, tuple):
                raise TypeError(
                    'key_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(keys))
                )
            jam.annotations.append(keys_to_jams(keys))

    # f0
    if f0_data is not None:
        if not isinstance(f0_data, list):
            raise TypeError('f0_data should be a list of tuples')
        for f0s in f0_data:
            if not isinstance(f0s, tuple):
                raise TypeError(
                    'f0_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(f0s))
                )
            jam.annotations.append(f0s_to_jams(f0s))

    # lyrics
    if lyrics_data is not None:
        if not isinstance(lyrics_data, list):
            raise TypeError('lyrics_data should be a list of tuples')
        for lyrics in lyrics_data:
            if not isinstance(lyrics, tuple):
                raise TypeError(
                    'lyrics_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(lyrics))
                )
            jam.annotations.append(lyrics_to_jams(lyrics))

    # tags
    if tags_gtzan_data is not None:
        if not isinstance(tags_gtzan_data, list):
            raise TypeError('tags_gtzan_data should be a list of tuples')
        for tag in tags_gtzan_data:
            if not isinstance(tag, tuple):
                raise TypeError(
                    'tags_gtzan_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(tag))
                )
            jam.annotations.append(tag_gtzan_to_jams(tag))

    # tag open
    if tags_open_data is not None:
        if not isinstance(tags_open_data, list):
            raise TypeError('tags_open_data should be a list of tuples')
        for tag in tags_open_data:
            if not isinstance(tag, tuple):
                raise TypeError(
                    'tags_open_data should be a list of tuples, '
                    + 'but contains a {} element'.format(type(tag))
                )
            jam.annotations.append(tag_open_to_jams(tag))

    return jam


def beats_to_jams(beats):
    """
    Convert beats annotations into jams format.

    Parameters
    ----------
    beats: tuple
        A tuple in the format (BeatData, str), where str describes the annotation
        and BeatData is the beats mirdata annotation format.

    Returns
    -------
    jannot_beat: JAM beat annotation object.

    """
    jannot_beat = jams.Annotation(namespace='beat')
    jannot_beat.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if beats[0] is not None:
        if not isinstance(beats[0], utils.BeatData):
            raise TypeError('Type should be BeatData.')
        for t, p in zip(beats[0].beat_times, beats[0].beat_positions):
            jannot_beat.append(time=t, duration=0.0, value=p)
    if beats[1] is not None:
        jannot_beat.sandbox = jams.Sandbox(name=beats[1])
    return jannot_beat


def sections_to_jams(sections):
    """
    Convert sections annotations into jams format.

    Parameters
    ----------
    sections: tuple
        A tuple in the format (SectionData, str), where str describes the annotation
        and SectionData is the sections mirdata annotation format.

    Returns
    -------
    jannot_seg: JAM segment_open annotation object.
    """
    jannot_seg = jams.Annotation(namespace='segment_open')
    jannot_seg.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if sections[0] is not None:
        if not isinstance(sections[0], utils.SectionData):
            raise TypeError('Type should be SectionData.')
        for inter, seg in zip(sections[0].intervals, sections[0].labels):
            jannot_seg.append(time=inter[0], duration=inter[1] - inter[0], value=seg)
    if sections[1] is not None:
        jannot_seg.sandbox = jams.Sandbox(name=sections[1])
    return jannot_seg


def chords_to_jams(chords):
    """
    Convert chords annotations into jams format.

    Parameters
    ----------
    chords: tuple
        A tuple in the format (ChordData, str), where str describes the annotation
        and ChordData is the chords mirdata annotation format.

    Returns
    -------
    jannot_chord: JAM chord annotation object.
    """
    jannot_chord = jams.Annotation(namespace='chord')
    jannot_chord.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if chords[0] is not None:
        if not isinstance(chords[0], utils.ChordData):
            raise TypeError('Type should be ChordData.')
        for beg, end, ch in zip(
            chords[0].intervals[:, 0], chords[0].intervals[:, 1], chords[0].labels
        ):
            jannot_chord.append(time=beg, duration=end - beg, value=ch)
    if chords[1] is not None:
        jannot_chord.sandbox = jams.Sandbox(name=chords[1])
    return jannot_chord


def notes_to_jams(notes):
    """
    Convert notes annotations into jams format using note_to_midi from librosa.

    Parameters
    ----------
    notes: tuple
        A tuple in the format (NoteData, str), where str describes the annotation
        and NoteData is the notes mirdata annotation format.

    Returns
    -------
    jannot_notes: JAM note_midi annotation object.
    """
    jannot_note = jams.Annotation(namespace='note_hz')
    jannot_note.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if notes[0] is not None:
        if not isinstance(notes[0], utils.NoteData):
            raise TypeError('Type should be NoteData.')
        for beg, end, n in zip(
            notes[0].intervals[:, 0], notes[0].intervals[:, 1], notes[0].notes
        ):
            jannot_note.append(time=beg, duration=end - beg, value=n)
    if notes[1] is not None:
        jannot_note.sandbox = jams.Sandbox(name=notes[1])
    return jannot_note


def keys_to_jams(keys):
    """
    Convert keys annotations into jams format.

    Parameters
    ----------
    keys: tuple
        A tuple in the format (KeyData, str), where str describes the annotation
        and KeyData is the keys mirdata annotation format.

    Returns
    -------
    jannot_key: JAM key_mode annotation object.
    """
    jannot_key = jams.Annotation(namespace='key_mode')
    jannot_key.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if keys[0] is not None:
        if not isinstance(keys[0], utils.KeyData):
            raise TypeError('Type should be KeyData.')
        for beg, end, key in zip(keys[0].start_times, keys[0].end_times, keys[0].keys):
            jannot_key.append(time=beg, duration=end - beg, value=key)
    if keys[1] is not None:
        jannot_key.sandbox = jams.Sandbox(name=keys[1])
    return jannot_key


def multi_sections_to_jams(multi_sections):
    """
    Convert hierarchical annotations into jams format.

    Parameters
    ----------
    multi_segment: list
        A list of tuples in the format ([(segments0, level0), (segments1, level1)], annotator),
        where segments are SectionData mirdata format, level indicates the hierarchy (e.g. 0, 1)
        and annotator describes the annotator. This format is customize for Salami dataset annotations.

    Returns
    -------
    jannot_multi: JAM multi_segment annotation object.
    """
    # sections with multiple annotators and multiple level annotations
    jannot_multi = jams.Annotation(namespace='multi_segment')
    jannot_multi.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    jannot_multi.annotation_metadata = jams.AnnotationMetadata(
        annotator={'name': multi_sections[1]}
    )
    for sections in multi_sections[0]:
        if sections[0] is not None:
            if not isinstance(sections[0], utils.SectionData):
                raise TypeError('Type should be SectionData.')
            for inter, seg in zip(sections[0].intervals, sections[0].labels):
                jannot_multi.append(
                    time=inter[0],
                    duration=inter[1] - inter[0],
                    value={'label': seg, 'level': sections[1]},
                )
    return jannot_multi


def tempos_to_jams(tempos):
    """
    Convert tempo annotations into jams format.

    Parameters
    ----------
    tempo: tuple
        A tuple in the format (float, str), where str describes the annotation
        and float is the tempo in beats per minute.

    Returns
    -------
    jannot_tempo: JAM tempo annotation object.
    """
    jannot_tempo = jams.Annotation(namespace='tempo')
    jannot_tempo.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if tempos[0] is not None:
        if not isinstance(tempos[0], float) and not isinstance(tempos[0], int):
            raise TypeError('Type should be float or int.')
        jannot_tempo.append(time=0, duration=0, confidence=1, value=tempos[0])
    if tempos[1] is not None:
        jannot_tempo.sandbox = jams.Sandbox(name=tempos[1])
    return jannot_tempo


def events_to_jams(events):
    """
    Convert events annotations into jams format.

    Parameters
    ----------
    events: tuple
        A tuple in the format (EventData, str), where str describes the annotation
        and EventData is the events mirdata annotation format.

    Returns
    -------
    jannot_events: JAM tag_open annotation object.
    """
    jannot_events = jams.Annotation(namespace='tag_open')
    jannot_events.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if events[0] is not None:
        if type(events[0]) != utils.EventData:
            raise TypeError('Type should be EventData.')
        for beg, end, label in zip(
            events[0].start_times, events[0].end_times, events[0].event
        ):
            jannot_events.append(time=beg, duration=end - beg, value=str(label))
    if events[1] is not None:
        jannot_events.sandbox = jams.Sandbox(name=events[1])
    return jannot_events


def f0s_to_jams(f0s):
    """
    Convert f0 annotations into jams format.

    Parameters
    ----------
    f0s: tuple
        A tuple in the format (F0Data, str), where str describes the annotation
        and F0Data is the f0 mirdata annotation format.

    Returns
    -------
    jannot_f0: JAM pitch_contour annotation object.
    """
    jannot_f0 = jams.Annotation(namespace='pitch_contour')
    jannot_f0.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if f0s[0] is not None:
        if not isinstance(f0s[0], utils.F0Data):
            raise TypeError('Type should be F0Data.')
        for t, f, c in zip(f0s[0].times, f0s[0].frequencies, f0s[0].confidence):
            jannot_f0.append(
                time=t,
                duration=0.0,
                value={'index': 0, 'frequency': f, 'voiced': f > 0},
                confidence=c,
            )
    if f0s[1] is not None:
        jannot_f0.sandbox = jams.Sandbox(name=f0s[1])
    return jannot_f0


def lyrics_to_jams(lyrics):
    """
    Convert lyrics annotations into jams format.

    Parameters
    ----------
    lyrics: tuple
        A tuple in the format (LyricData, str), where str describes the annotation
        and LyricData is the lyric mirdata annotation format.

    Returns
    -------
    jannot_lyric: JAM lyric annotation object.
    """
    jannot_lyric = jams.Annotation(namespace='lyrics')
    jannot_lyric.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if lyrics[0] is not None:
        if not isinstance(lyrics[0], utils.LyricData):
            raise TypeError('Type should be LyricData.')
        for beg, end, lyric in zip(
            lyrics[0].start_times, lyrics[0].end_times, lyrics[0].lyrics
        ):
            jannot_lyric.append(time=beg, duration=end - beg, value=lyric)
    if lyrics[1] is not None:
        jannot_lyric.sandbox = jams.Sandbox(name=lyrics[1])
    return jannot_lyric


def tag_gtzan_to_jams(tags):
    """
    Convert tag-gtzan annotations into jams format.

    Parameters
    ----------
    tags: tuple
        A tuple in the format (str, str), where the first str is the tag
        and the second describes the annotation.

    Returns
    -------
    jannot_tag_gtzan: JAM tag_gtzan annotation object.
    """
    jannot_tag_gtzan = jams.Annotation(namespace='tag_gtzan')
    jannot_tag_gtzan.annotation_metadata = jams.AnnotationMetadata(
        data_source='mirdata'
    )
    if tags[0] is not None:
        if not isinstance(tags[0], str):
            raise TypeError('Type should be str.')
        jannot_tag_gtzan.append(time=0.0, duration=0.0, value=tags[0])
    if tags[1] is not None:
        jannot_tag_gtzan.sandbox = jams.Sandbox(name=tags[1])
    return jannot_tag_gtzan


def tag_open_to_jams(tags):
    """
    Convert tag-open annotations into jams format.

    Parameters
    ----------
    tags: tuple
        A tuple in the format (str, str), where the first str is the open tag
        and the second describes the annotation.

    Returns
    -------
    jannot_tag_open: JAM tag_open annotation object.
    """
    jannot_tag_open = jams.Annotation(namespace='tag_open')
    jannot_tag_open.annotation_metadata = jams.AnnotationMetadata(data_source='mirdata')
    if tags[0] is not None:
        if not isinstance(tags[0], str):
            raise TypeError('Type should be str.')
        jannot_tag_open.append(time=0.0, duration=0.0, value=tags[0])
    if tags[1] is not None:
        jannot_tag_open.sandbox = jams.Sandbox(name=tags[1])
    return jannot_tag_open



# -*- coding: utf-8 -*-

import numpy as np
import pytest
import jams

from mirdata import jams_utils, utils


def get_jam_data(jam, namespace, annot_numb):
    time = []
    duration = []
    value = []
    confidence = []
    for obs in jam.search(namespace=namespace)[annot_numb]['data']:
        time.append(obs.time)
        duration.append(round(obs.duration, 3))
        value.append(obs.value)
        confidence.append(obs.confidence)
    return time, duration, value, confidence


def test_beats():
    beat_data_1 = [(utils.BeatData(np.array([0.2, 0.3]), np.array([1, 2])), None)]
    beat_data_2 = [(utils.BeatData(np.array([0.5, 0.7]), np.array([2, 3])), 'beats_2')]
    beat_data_3 = [
        (utils.BeatData(np.array([0.0, 0.3]), np.array([1, 2])), 'beats_1'),
        (utils.BeatData(np.array([0.5, 0.13]), np.array([4, 3])), 'beats_2'),
    ]
    beat_data_4 = (utils.BeatData(np.array([0.0, 0.3]), np.array([1, 2])), 'beats_1')
    beat_data_5 = [
        (utils.BeatData(np.array([0.0, 0.3]), np.array([1, 2])), 'beats_1'),
        [utils.BeatData(np.array([0.5, 0.13]), np.array([4, 3])), 'beats_2'],
    ]
    beat_data_6 = [(None, None)]
    beat_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(beat_data=beat_data_1)
    jam_2 = jams_utils.jams_converter(beat_data=beat_data_2)
    jam_3 = jams_utils.jams_converter(beat_data=beat_data_3)
    jam_6 = jams_utils.jams_converter(beat_data=beat_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'beat', 0)
    assert time == [0.2, 0.3]
    assert duration == [0.0, 0.0]
    assert value == [1, 2]
    assert confidence == [None, None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'beats_2'

    time, duration, value, confidence = get_jam_data(jam_3, 'beat', 0)
    assert time == [0.0, 0.3]
    assert duration == [0.0, 0.0]
    assert value == [1, 2]
    assert confidence == [None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'beat', 1)
    assert time == [0.13, 0.5]
    assert duration == [0.0, 0.0]
    assert value == [3, 4]
    assert confidence == [None, None]

    time, duration, value, confidence = get_jam_data(jam_6, 'beat', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(beat_data=beat_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(beat_data=beat_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(beat_data=beat_data_7)


def test_chords():
    chord_data_1 = [
        (
            utils.ChordData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array(['A', 'A', 'E']),
            ),
            None,
        )
    ]
    chord_data_2 = [
        (
            utils.ChordData(
                np.array([[0.0, 0.8, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array(['A', 'B', 'C']),
            ),
            'chords_2',
        )
    ]
    chord_data_3 = [
        (
            utils.ChordData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array(['A', 'A', 'E']),
            ),
            'chords_1',
        ),
        (
            utils.ChordData(
                np.array([[0.0, 0.7, 1.0], [0.7, 1.0, 1.5]]).T,
                np.array(['A', 'B', 'C']),
            ),
            'chords_2',
        ),
    ]
    chord_data_4 = (
        utils.ChordData(
            np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T, np.array(['A', 'A', 'E'])
        ),
        None,
    )
    chord_data_5 = [
        [
            utils.ChordData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array(['A', 'A', 'E']),
            ),
            None,
        ],
        (
            utils.ChordData(
                np.array([[0.0, 0.8, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array(['A', 'B', 'C']),
            ),
            'chords_2',
        ),
    ]
    chord_data_6 = [(None, None)]
    chord_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(chord_data=chord_data_1)
    jam_2 = jams_utils.jams_converter(chord_data=chord_data_2)
    jam_3 = jams_utils.jams_converter(chord_data=chord_data_3)
    jam_6 = jams_utils.jams_converter(chord_data=chord_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'chord', 0)
    assert time == [0.0, 0.5, 1.0]
    assert duration == [0.5, 0.5, 0.5]
    assert value == ['A', 'A', 'E']
    assert confidence == [None, None, None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'chords_2'

    time, duration, value, confidence = get_jam_data(jam_3, 'chord', 0)
    assert time == [0.0, 0.5, 1.0]
    assert duration == [0.5, 0.5, 0.5]
    assert value == ['A', 'A', 'E']
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'chord', 1)
    assert time == [0.0, 0.7, 1.0]
    assert duration == [0.7, 0.3, 0.5]
    assert value == ['A', 'B', 'C']
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_6, 'chord', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(chord_data=chord_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(chord_data=chord_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(chord_data=chord_data_7)


def test_notes():
    note_data_1 = [
        (
            utils.NoteData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            None,
        )
    ]
    note_data_2 = [
        (
            utils.NoteData(
                np.array([[0.0, 0.8, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            'notes_2',
        )
    ]
    note_data_3 = [
        (
            utils.NoteData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            'notes_1',
        ),
        (
            utils.NoteData(
                np.array([[0.0, 0.7, 1.0], [0.7, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            'notes_2',
        ),
    ]
    note_data_4 = (
        utils.NoteData(
            np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
            np.array([1108.731, 1108.731, 1108.731]),
            np.array([1, 1, 1]),
        ),
        None,
    )
    note_data_5 = [
        [
            utils.NoteData(
                np.array([[0.0, 0.5, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            None,
        ],
        (
            utils.NoteData(
                np.array([[0.0, 0.8, 1.0], [0.5, 1.0, 1.5]]).T,
                np.array([1108.731, 1108.731, 1108.731]),
                np.array([1, 1, 1]),
            ),
            'notes_2',
        ),
    ]
    note_data_6 = [(None, None)]
    note_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(note_data=note_data_1)
    jam_2 = jams_utils.jams_converter(note_data=note_data_2)
    jam_3 = jams_utils.jams_converter(note_data=note_data_3)
    jam_6 = jams_utils.jams_converter(note_data=note_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'note_hz', 0)
    assert time == [0.0, 0.5, 1.0]
    assert duration == [0.5, 0.5, 0.5]
    assert value == [1108.731, 1108.731, 1108.731]
    assert confidence == [None, None, None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'notes_2'

    time, duration, value, confidence = get_jam_data(jam_3, 'note_hz', 0)
    assert time == [0.0, 0.5, 1.0]
    assert duration == [0.5, 0.5, 0.5]
    assert value == [1108.731, 1108.731, 1108.731]
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'note_hz', 1)
    assert time == [0.0, 0.7, 1.0]
    assert duration == [0.7, 0.3, 0.5]
    assert value == [1108.731, 1108.731, 1108.731]
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_6, 'note_hz', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(note_data=note_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(note_data=note_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(note_data=note_data_7)


def test_sections():
    section_data_1 = [
        (
            utils.SectionData(
                np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                np.array(['verse A', 'verse B', 'verse A']),
            ),
            None,
        )
    ]
    section_data_2 = [
        (
            utils.SectionData(
                np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                np.array(['verse A', 'verse B', 'verse A']),
            ),
            'sections_2',
        )
    ]
    section_data_3 = [
        (
            utils.SectionData(
                np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                np.array(['verse A', 'verse B', 'verse A']),
            ),
            'sections_1',
        ),
        (
            utils.SectionData(
                np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 30.0]]).T,
                np.array(['verse A', 'verse B', 'verse C']),
            ),
            'sections_2',
        ),
    ]
    section_data_4 = (
        utils.SectionData(
            np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
            np.array(['verse A', 'verse B', 'verse A']),
        ),
        None,
    )
    section_data_5 = [
        [
            utils.SectionData(
                np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                np.array(['verse A', 'verse B', 'verse A']),
            ),
            None,
        ],
        (
            utils.SectionData(
                np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                np.array(['verse A', 'verse B', 'verse A']),
            ),
            'sections_2',
        ),
    ]
    section_data_6 = [(None, None)]
    section_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(section_data=section_data_1)
    jam_2 = jams_utils.jams_converter(section_data=section_data_2)
    jam_3 = jams_utils.jams_converter(section_data=section_data_3)
    jam_6 = jams_utils.jams_converter(section_data=section_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'segment', 0)
    assert time == [0.0, 10.0, 20.0]
    assert duration == [10.0, 10.0, 5.0]
    assert value == ['verse A', 'verse B', 'verse A']
    assert confidence == [None, None, None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'sections_2'

    time, duration, value, confidence = get_jam_data(jam_3, 'segment', 0)
    assert time == [0.0, 10.0, 20.0]
    assert duration == [10.0, 10.0, 5.0]
    assert value == ['verse A', 'verse B', 'verse A']
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'segment', 1)
    assert time == [0.0, 15.0, 20.0]
    assert duration == [15.0, 5.0, 10.0]
    assert value == ['verse A', 'verse B', 'verse C']
    assert confidence == [None, None, None]

    time, duration, value, confidence = get_jam_data(jam_6, 'segment', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(section_data=section_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(section_data=section_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(section_data=section_data_7)


def test_multi_sections():
    multi_section_data_1 = [
        (
            [
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    None,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    None,
                ),
            ],
            None,
        )
    ]

    multi_section_data_2 = [
        (
            [
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    0,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    1,
                ),
            ],
            'annotator_1',
        )
    ]
    multi_section_data_3 = [
        (
            [
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    0,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    1,
                ),
            ],
            'annotator_1',
        ),
        (
            [
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    0,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    1,
                ),
            ],
            'annotator_2',
        ),
    ]
    multi_section_data_4 = (
        [
            (
                utils.SectionData(
                    np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                    np.array(['verse A', 'verse B', 'verse A']),
                ),
                None,
            ),
            (
                utils.SectionData(
                    np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                    np.array(['verse a', 'verse b', 'verse a']),
                ),
                None,
            ),
        ],
        None,
    )
    multi_section_data_5 = [
        [
            [
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    None,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    None,
                ),
            ],
            None,
        ]
    ]
    multi_section_data_6 = [
        (
            (
                (
                    utils.SectionData(
                        np.array([[0.0, 10.0, 20.0], [10.0, 20.0, 25.0]]).T,
                        np.array(['verse A', 'verse B', 'verse A']),
                    ),
                    None,
                ),
                (
                    utils.SectionData(
                        np.array([[0.0, 15.0, 20.0], [15.0, 20.0, 25.0]]).T,
                        np.array(['verse a', 'verse b', 'verse a']),
                    ),
                    None,
                ),
            ),
            None,
        )
    ]
    multi_section_data_7 = [([(None, None), (None, None)], None)]
    multi_section_data_8 = [
        (
            [
                (
                    utils.EventData(
                        np.array([0.2, 0.3]),
                        np.array([0.3, 0.4]),
                        np.array(['event A', 'event B']),
                    ),
                    None,
                ),
                (
                    utils.EventData(
                        np.array([0.2, 0.3]),
                        np.array([0.3, 0.4]),
                        np.array(['event A', 'event B']),
                    ),
                    None,
                ),
            ],
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(multi_section_data=multi_section_data_1)
    jam_2 = jams_utils.jams_converter(multi_section_data=multi_section_data_2)
    jam_3 = jams_utils.jams_converter(multi_section_data=multi_section_data_3)
    jam_7 = jams_utils.jams_converter(multi_section_data=multi_section_data_7)

    time, duration, value, confidence = get_jam_data(jam_1, 'multi_segment', 0)
    assert time == [0.0, 0.0, 10.0, 15.0, 20.0, 20.0]
    assert duration == [10.0, 15.0, 10.0, 5.0, 5.0, 5.0]
    assert value == [
        {'label': 'verse A', 'level': None},
        {'label': 'verse a', 'level': None},
        {'label': 'verse B', 'level': None},
        {'label': 'verse b', 'level': None},
        {'label': 'verse A', 'level': None},
        {'label': 'verse a', 'level': None},
    ]
    assert confidence == [None, None, None, None, None, None]

    assert (
        jam_2.annotations[0]['annotation_metadata']['annotator']['name']
        == 'annotator_1'
    )

    time, duration, value, confidence = get_jam_data(jam_3, 'multi_segment', 0)
    assert time == [0.0, 0.0, 10.0, 15.0, 20.0, 20.0]
    assert duration == [10.0, 15.0, 10.0, 5.0, 5.0, 5.0]
    assert value == [
        {'label': 'verse A', 'level': 0},
        {'label': 'verse a', 'level': 1},
        {'label': 'verse B', 'level': 0},
        {'label': 'verse b', 'level': 1},
        {'label': 'verse A', 'level': 0},
        {'label': 'verse a', 'level': 1},
    ]
    assert confidence == [None, None, None, None, None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'multi_segment', 1)
    assert time == [0.0, 0.0, 10.0, 15.0, 20.0, 20.0]
    assert duration == [10.0, 15.0, 10.0, 5.0, 5.0, 5.0]
    assert value == [
        {'label': 'verse A', 'level': 0},
        {'label': 'verse a', 'level': 1},
        {'label': 'verse B', 'level': 0},
        {'label': 'verse b', 'level': 1},
        {'label': 'verse A', 'level': 0},
        {'label': 'verse a', 'level': 1},
    ]
    assert confidence == [None, None, None, None, None, None]

    time, duration, value, confidence = get_jam_data(jam_7, 'multi_segment', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(multi_section_data=multi_section_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(multi_section_data=multi_section_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(multi_section_data=multi_section_data_6)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(multi_section_data=multi_section_data_8)


def test_keys():
    key_data_1 = [
        (utils.KeyData(np.array([0.0]), np.array([100.0]), np.array(['A'])), None)
    ]
    key_data_2 = [
        (utils.KeyData(np.array([0.0]), np.array([100.0]), np.array(['A'])), 'keys_1')
    ]
    key_data_3 = [
        (utils.KeyData(np.array([0.0]), np.array([100.0]), np.array(['A'])), 'keys_1'),
        (utils.KeyData(np.array([0.0]), np.array([50.0]), np.array(['B'])), 'keys_2'),
    ]
    key_data_4 = (
        utils.KeyData(np.array([0.0]), np.array([100.0]), np.array(['A'])),
        'keys_1',
    )
    key_data_5 = [
        [utils.KeyData(np.array([0.0]), np.array([100.0]), np.array(['A'])), 'keys_1'],
        (utils.KeyData(np.array([0.0]), np.array([50.0]), np.array(['B'])), 'keys_2'),
    ]
    key_data_6 = [(None, None)]
    key_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(key_data=key_data_1)
    jam_2 = jams_utils.jams_converter(key_data=key_data_2)
    jam_3 = jams_utils.jams_converter(key_data=key_data_3)
    jam_6 = jams_utils.jams_converter(key_data=key_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'key', 0)
    assert time == [0.0]
    assert duration == [100.0]
    assert value == ['A']
    assert confidence == [None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'keys_1'

    time, duration, value, confidence = get_jam_data(jam_3, 'key', 0)
    assert time == [0.0]
    assert duration == [100.0]
    assert value == ['A']
    assert confidence == [None]

    time, duration, value, confidence = get_jam_data(jam_3, 'key', 1)
    assert time == [0.0]
    assert duration == [50.0]
    assert value == ['B']
    assert confidence == [None]

    time, duration, value, confidence = get_jam_data(jam_6, 'key', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(key_data=key_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(key_data=key_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(key_data=key_data_7)


def test_f0s():
    f0_data_1 = [
        (
            utils.F0Data(
                np.array([0.016, 0.048]), np.array([0.0, 260.9]), np.array([0.0, 1.0])
            ),
            None,
        )
    ]
    f0_data_2 = [
        (
            utils.F0Data(
                np.array([0.016, 0.048]), np.array([0.0, 260.9]), np.array([0.0, 1.0])
            ),
            'f0s_1',
        )
    ]
    f0_data_3 = [
        (
            utils.F0Data(
                np.array([0.016, 0.048]), np.array([0.0, 260.9]), np.array([0.0, 1.0])
            ),
            'f0s_1',
        ),
        (
            utils.F0Data(
                np.array([0.003, 0.012]), np.array([0.0, 230.5]), np.array([0.0, 1.0])
            ),
            'f0s_2',
        ),
    ]
    f0_data_4 = (
        utils.F0Data(
            np.array([0.016, 0.048]), np.array([0.0, 260.9]), np.array([0.0, 1.0])
        ),
        'f0s_1',
    )
    f0_data_5 = [
        [
            utils.F0Data(
                np.array([0.016, 0.048]), np.array([0.0, 260.9]), np.array([0.0, 1.0])
            ),
            'f0s_1',
        ],
        (
            utils.F0Data(
                np.array([0.003, 0.012]), np.array([0.0, 230.5]), np.array([0.0, 1.0])
            ),
            'f0s_2',
        ),
    ]
    f0_data_6 = [(None, None)]
    f0_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(f0_data=f0_data_1)
    jam_2 = jams_utils.jams_converter(f0_data=f0_data_2)
    jam_3 = jams_utils.jams_converter(f0_data=f0_data_3)
    jam_6 = jams_utils.jams_converter(f0_data=f0_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'pitch_contour', 0)
    assert time == [0.016, 0.048]
    assert duration == [0.0, 0.0]
    assert value == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 260.9, 'index': 0, 'voiced': True},
    ]
    assert confidence == [0.0, 1.0]

    assert jam_2.annotations[0]['sandbox']['name'] == 'f0s_1'

    time, duration, value, confidence = get_jam_data(jam_3, 'pitch_contour', 0)
    assert time == [0.016, 0.048]
    assert duration == [0.0, 0.0]
    assert value == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 260.9, 'index': 0, 'voiced': True},
    ]
    assert confidence == [0.0, 1.0]

    time, duration, value, confidence = get_jam_data(jam_3, 'pitch_contour', 1)
    assert time == [0.003, 0.012]
    assert duration == [0.0, 0.0]
    assert value == [
        {'frequency': 0.0, 'index': 0, 'voiced': False},
        {'frequency': 230.5, 'index': 0, 'voiced': True},
    ]
    assert confidence == [0.0, 1.0]

    time, duration, value, confidence = get_jam_data(jam_6, 'pitch_contour', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(f0_data=f0_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(f0_data=f0_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(f0_data=f0_data_7)


def test_lyrics():
    lyrics_data_1 = [
        (
            utils.LyricData(
                np.array([0.027, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['The', 'Test']),
                np.array([None, None]),
            ),
            None,
        )
    ]
    lyrics_data_2 = [
        (
            utils.LyricData(
                np.array([0.027, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['The', 'Test']),
                np.array([None, None]),
            ),
            'lyrics_1',
        )
    ]
    lyrics_data_3 = [
        (
            utils.LyricData(
                np.array([0.027, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['The', 'Test']),
                np.array([None, None]),
            ),
            'lyrics_1',
        ),
        (
            utils.LyricData(
                np.array([0.0, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['is', 'cool']),
                np.array([None, None]),
            ),
            'lyrics_2',
        ),
    ]
    lyrics_data_4 = (
        utils.LyricData(
            np.array([0.027, 0.232]),
            np.array([0.227, 0.742]),
            np.array(['The', 'Test']),
            np.array([None, None]),
        ),
        'lyrics_1',
    )
    lyrics_data_5 = [
        (
            utils.LyricData(
                np.array([0.027, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['The', 'Test']),
                np.array([None, None]),
            ),
            'lyrics_1',
        ),
        [
            utils.LyricData(
                np.array([0.0, 0.232]),
                np.array([0.227, 0.742]),
                np.array(['is', 'cool']),
                np.array([None, None]),
            ),
            'lyrics_2',
        ],
    ]
    lyrics_data_6 = [(None, None)]
    lyrics_data_7 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            None,
        )
    ]

    jam_1 = jams_utils.jams_converter(lyrics_data=lyrics_data_1)
    jam_2 = jams_utils.jams_converter(lyrics_data=lyrics_data_2)
    jam_3 = jams_utils.jams_converter(lyrics_data=lyrics_data_3)
    jam_6 = jams_utils.jams_converter(lyrics_data=lyrics_data_6)

    time, duration, value, confidence = get_jam_data(jam_1, 'lyrics', 0)
    assert time == [0.027, 0.232]
    assert duration == [0.2, 0.51]
    assert value == ['The', 'Test']
    assert confidence == [None, None]

    assert jam_2.annotations[0]['sandbox']['name'] == 'lyrics_1'

    time, duration, value, confidence = get_jam_data(jam_3, 'lyrics', 0)
    assert time == [0.027, 0.232]
    assert duration == [0.2, 0.51]
    assert value == ['The', 'Test']
    assert confidence == [None, None]

    time, duration, value, confidence = get_jam_data(jam_3, 'lyrics', 1)
    assert time == [0.0, 0.232]
    assert duration == [0.227, 0.51]
    assert value == ['is', 'cool']
    assert confidence == [None, None]

    time, duration, value, confidence = get_jam_data(jam_6, 'lyrics', 0)
    assert time == []
    assert duration == []
    assert value == []
    assert confidence == []

    assert type(jam_1) == jams.JAMS

    with pytest.raises(TypeError):
        jams_utils.jams_converter(lyrics_data=lyrics_data_4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(lyrics_data=lyrics_data_5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(lyrics_data=lyrics_data_7)


def test_tags():
    tag_data1 = [('blues', 'I am a description')]
    tag_data2 = [('disco', 'tag 1'), ('rock', 'tag 2')]
    tag_data3 = [('invalid', 'asdf')]
    tag_data4 = ('jazz', 'wrong format')
    tag_data5 = ['wrong format too']
    tag_data6 = [(123, 'asdf')]
    jam1 = jams_utils.jams_converter(
        tags_gtzan_data=tag_data1, metadata={'duration': 10.0}
    )
    assert jam1.validate()
    jam2 = jams_utils.jams_converter(
        tags_gtzan_data=tag_data2, metadata={'duration': 10.0}
    )
    assert jam2.validate()
    jam3 = jams_utils.jams_converter(
        tags_gtzan_data=tag_data3, metadata={'duration': 10.0}
    )
    with pytest.raises(jams.SchemaError):
        assert jam3.validate()
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tags_gtzan_data=tag_data4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tags_gtzan_data=tag_data5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tags_gtzan_data=tag_data6)


def test_tempos():
    tempo_data1 = [(120, 'I am a description')]
    tempo_data2 = [(120.0, 'tempo 1'), (240, 'tempo 2')]
    tempo_data3 = [(-1, 'asdf')]
    tempo_data4 = (120.5, 'wrong format')
    tempo_data5 = ['wrong format too']
    tempo_data6 = [('string!', 'string!')]
    jam1 = jams_utils.jams_converter(
        tempo_data=tempo_data1, metadata={'duration': 10.0}
    )
    assert jam1.validate()
    jam2 = jams_utils.jams_converter(
        tempo_data=tempo_data2, metadata={'duration': 10.0}
    )
    assert jam2.validate()
    jam3 = jams_utils.jams_converter(
        tempo_data=tempo_data3, metadata={'duration': 10.0}
    )
    with pytest.raises(jams.SchemaError):
        assert jam3.validate()
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tempo_data=tempo_data4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tempo_data=tempo_data5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(tempo_data=tempo_data6)


def test_events():
    event_data1 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array(['event A', 'event B']),
            ),
            'I am a description',
        )
    ]
    event_data2 = [
        (
            utils.EventData(
                np.array([0.2, 0.3]), np.array([0.4, 0.5]), np.array([2, 'event B'])
            ),
            'events 1',
        ),
        (
            utils.EventData(
                np.array([0.2, 0.3]),
                np.array([0.3, 0.4]),
                np.array([{'a': 1, 2: 'b'}, 'a great label']),
            ),
            'events 2',
        ),
    ]
    event_data3 = [
        (
            utils.EventData(
                np.array([20, 30]),  # invalid because > duration
                np.array([0.3, 0.4]),
                np.array([{'a': 1, 2: 'b'}, 'a great label']),
            ),
            'asdf',
        )
    ]
    event_data4 = ('jazz', 'wrong format')
    event_data5 = ['wrong format too']
    event_data6 = [('wrong', 'description')]
    jam1 = jams_utils.jams_converter(
        event_data=event_data1, metadata={'duration': 10.0}
    )
    assert jam1.validate()
    jam2 = jams_utils.jams_converter(
        event_data=event_data2, metadata={'duration': 10.0}
    )
    assert jam2.validate()
    jam3 = jams_utils.jams_converter(
        event_data=event_data3, metadata={'duration': 10.0}
    )
    with pytest.raises(jams.SchemaError):
        assert jam3.validate()
    with pytest.raises(TypeError):
        jams_utils.jams_converter(event_data=event_data4)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(event_data=event_data5)
    with pytest.raises(TypeError):
        jams_utils.jams_converter(event_data=event_data6)


def test_metadata():
    metadata_1 = {
        'duration': 1.5,
        'artist': 'Meatloaf',
        'title': 'Le ciel est blue',
        'favourite_color': 'rainbow',
    }

    jam_1 = jams_utils.jams_converter(lyrics_data=[(None, None)], metadata=metadata_1)

    assert jam_1['file_metadata']['title'] == 'Le ciel est blue'
    assert jam_1['file_metadata']['artist'] == 'Meatloaf'
    assert jam_1['file_metadata']['duration'] == 1.5
    assert jam_1['sandbox']['favourite_color'] == 'rainbow'

    # test meatadata value None
    metadata_2 = {
        'duration': 1.5,
        'artist': 'breakmaster cylinder',
        'title': None,
        'extra': None,
    }
    jam2 = jams_utils.jams_converter(metadata=metadata_2)
    assert jam2.validate()
    assert jam2['file_metadata']['duration'] == 1.5
    assert jam2['file_metadata']['artist'] == 'breakmaster cylinder'
    assert jam2['file_metadata']['title'] == ''
    assert 'extra' not in jam2['sandbox']


def test_duration():
    # duration from audio file
    jam = jams_utils.jams_converter(
        audio_path='tests/resources/mir_datasets/iKala/Wavfile/10161_chorus.wav'
    )
    assert jam.file_metadata.duration == 2.0
    assert jam.validate()

    # test invalid file path
    with pytest.raises(OSError):
        jams_utils.jams_converter(audio_path='i/dont/exist')

    jam1 = jams_utils.jams_converter(metadata={'duration': 4})
    assert jam1.file_metadata.duration == 4.0
    assert jam1.validate()

    # test incomplete metadata
    jam2 = jams_utils.jams_converter(metadata={'artist': 'b'})
    with pytest.raises(jams_utils.jams.SchemaError):
        jam2.validate()

    # test metadata duration and audio file equal
    jam3 = jams_utils.jams_converter(
        audio_path='tests/resources/mir_datasets/iKala/Wavfile/10161_chorus.wav',
        metadata={'duration': 2},
    )
    assert jam3.file_metadata.duration == 2
    assert jam3.validate()

    # test metadata and duration not equal
    jam4 = jams_utils.jams_converter(
        audio_path='tests/resources/mir_datasets/iKala/Wavfile/10161_chorus.wav',
        metadata={'duration': 1000},
    )
    assert jam4.file_metadata.duration == 1000
    assert jam4.validate()



# -*- coding: utf-8 -*-
"""Beatles Dataset Loader

The Beatles Dataset includes beat and metric position, chord, key, and segmentation
annotations for 179 Beatles songs. Details can be found in http://matthiasmauch.net/_pdf/mauch_omp_2009.pdf and
http://isophonics.net/content/reference-annotations-beatles.

"""

import csv
import librosa
import numpy as np
import os

from mirdata import download_utils
from mirdata import jams_utils
from mirdata import track
from mirdata import utils

DATASET_DIR = 'Beatles'
REMOTES = {
    'annotations': download_utils.RemoteFileMetadata(
        filename='The Beatles Annotations.tar.gz',
        url='http://isophonics.net/files/annotations/The%20Beatles%20Annotations.tar.gz',
        checksum='62425c552d37c6bb655a78e4603828cc',
        destination_dir='annotations',
    )
}

DATA = utils.LargeData('beatles_index.json')


class Track(track.Track):
    """Beatles track class

    Args:
        track_id (str): track id of the track
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Attributes:
        audio_path (str): track audio path
        beats_path (str): beat annotation path
        chords_path (str): chord annotation path
        keys_path (str): key annotation path
        sections_path (str): sections annotation path
        title (str): title of the track
        track_id (str): track id

    """

    def __init__(self, track_id, data_home=None):
        if track_id not in DATA.index:
            raise ValueError('{} is not a valid track ID in Beatles'.format(track_id))

        self.track_id = track_id

        if data_home is None:
            data_home = utils.get_default_dataset_path(DATASET_DIR)

        self._data_home = data_home
        self._track_paths = DATA.index[track_id]
        self.beats_path = utils.none_path_join(
            [self._data_home, self._track_paths['beat'][0]]
        )
        self.chords_path = os.path.join(self._data_home, self._track_paths['chords'][0])
        self.keys_path = utils.none_path_join(
            [self._data_home, self._track_paths['keys'][0]]
        )
        self.sections_path = os.path.join(
            self._data_home, self._track_paths['sections'][0]
        )
        self.audio_path = os.path.join(self._data_home, self._track_paths['audio'][0])

        self.title = os.path.basename(self._track_paths['sections'][0]).split('.')[0]

    @utils.cached_property
    def beats(self):
        """BeatData: human-labeled beat annotation"""
        return load_beats(self.beats_path)

    @utils.cached_property
    def chords(self):
        """ChordData: chord annotation"""
        return load_chords(self.chords_path)

    @utils.cached_property
    def key(self):
        """KeyData: key annotation"""
        return load_key(self.keys_path)

    @utils.cached_property
    def sections(self):
        """SectionData: section annotation"""
        return load_sections(self.sections_path)

    @property
    def audio(self):
        """(np.ndarray, float): audio signal, sample rate"""
        return load_audio(self.audio_path)

    def to_jams(self):
        """Jams: the track's data in jams format"""
        return jams_utils.jams_converter(
            audio_path=self.audio_path,
            beat_data=[(self.beats, None)],
            section_data=[(self.sections, None)],
            chord_data=[(self.chords, None)],
            key_data=[(self.key, None)],
            metadata={'artist': 'The Beatles', 'title': self.title},
        )


def load_audio(audio_path):
    """Load a Beatles audio file.

    Args:
        audio_path (str): path to audio file

    Returns:
        y (np.ndarray): the mono audio signal
        sr (float): The sample rate of the audio file

    """
    if not os.path.exists(audio_path):
        raise IOError("audio_path {} does not exist".format(audio_path))
    return librosa.load(audio_path, sr=None, mono=True)


def download(data_home=None, force_overwrite=False, cleanup=True):
    """Download the Beatles Dataset (annotations).
    The audio files are not provided due to copyright issues.

    Args:
        data_home (str):
            Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`
        force_overwrite (bool):
            Whether to overwrite the existing downloaded data
        cleanup (bool):
            Whether to delete the zip/tar file after extracting.

    """

    # use the default location: ~/mir_datasets/Beatles
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    download_message = """
        Unfortunately the audio files of the Beatles dataset are not available
        for download. If you have the Beatles dataset, place the contents into
        a folder called Beatles with the following structure:
            > Beatles/
                > annotations/
                > audio/
        and copy the Beatles folder to {}
    """.format(
        data_home
    )

    download_utils.downloader(
        data_home,
        remotes=REMOTES,
        info_message=download_message,
        force_overwrite=force_overwrite,
        cleanup=cleanup,
    )


def validate(data_home=None, silence=False):
    """Validate if a local version of this dataset is consistent

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        missing_files (list): List of file paths that are in the dataset index
            but missing locally
        invalid_checksums (list): List of file paths where the expected file exists locally
            but has a different checksum than the reference

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    missing_files, invalid_checksums = utils.validator(
        DATA.index, data_home, silence=silence
    )
    return missing_files, invalid_checksums


def track_ids():
    """Get the list of track IDs for this dataset

    Returns:
        (list): A list of track ids
    """
    return list(DATA.index.keys())


def load(data_home=None):
    """Load Beatles dataset

    Args:
        data_home (str): Local path where the dataset is stored.
            If `None`, looks for the data in the default directory, `~/mir_datasets`

    Returns:
        (dict): {`track_id`: track data}

    """
    if data_home is None:
        data_home = utils.get_default_dataset_path(DATASET_DIR)

    beatles_data = {}
    for key in track_ids():
        beatles_data[key] = Track(key, data_home=data_home)
    return beatles_data


def load_beats(beats_path):
    """Load Beatles format beat data from a file

    Args:
        beats_path (str): path to beat annotation file

    Returns:
        (utils.BeatData): loaded beat data

    """
    if beats_path is None:
        return None

    if not os.path.exists(beats_path):
        raise IOError("beats_path {} does not exist".format(beats_path))

    beat_times, beat_positions = [], []
    with open(beats_path, 'r') as fhandle:
        dialect = csv.Sniffer().sniff(fhandle.read(1024))
        fhandle.seek(0)
        reader = csv.reader(fhandle, dialect)
        for line in reader:
            beat_times.append(float(line[0]))
            beat_positions.append(line[-1])

    beat_positions = _fix_newpoint(np.array(beat_positions))
    # After fixing New Point labels convert positions to int
    beat_positions = [int(b) for b in beat_positions]

    beat_data = utils.BeatData(np.array(beat_times), np.array(beat_positions))

    return beat_data


def load_chords(chords_path):
    """Load Beatles format chord data from a file

    Args:
        chords_path (str): path to chord annotation file

    Returns:
        (utils.ChordData): loaded chord data

    """
    if chords_path is None:
        return None

    if not os.path.exists(chords_path):
        raise IOError("chords_path {} does not exist".format(chords_path))

    start_times, end_times, chords = [], [], []
    with open(chords_path, 'r') as f:
        dialect = csv.Sniffer().sniff(f.read(1024))
        f.seek(0)
        reader = csv.reader(f, dialect)
        for line in reader:
            start_times.append(float(line[0]))
            end_times.append(float(line[1]))
            chords.append(line[2])

    chord_data = utils.ChordData(np.array([start_times, end_times]).T, chords)

    return chord_data


def load_key(keys_path):
    """Load Beatles format key data from a file

    Args:
        keys_path (str): path to key annotation file

    Returns:
        (utils.KeyData): loaded key data

    """
    if keys_path is None:
        return None

    if not os.path.exists(keys_path):
        raise IOError("keys_path {} does not exist".format(keys_path))

    start_times, end_times, keys = [], [], []
    with open(keys_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            if line[2] == 'Key':
                start_times.append(float(line[0]))
                end_times.append(float(line[1]))
                keys.append(line[3])

    key_data = utils.KeyData(np.array(start_times), np.array(end_times), np.array(keys))

    return key_data


def load_sections(sections_path):
    """Load Beatles format section data from a file

    Args:
        sections_path (str): path to section annotation file

    Returns:
        (utils.SectionData): loaded section data

    """
    if sections_path is None:
        return None

    if not os.path.exists(sections_path):
        raise IOError("sections_path {} does not exist".format(sections_path))

    start_times, end_times, sections = [], [], []
    with open(sections_path, 'r') as fhandle:
        reader = csv.reader(fhandle, delimiter='\t')
        for line in reader:
            start_times.append(float(line[0]))
            end_times.append(float(line[1]))
            sections.append(line[3])

    section_data = utils.SectionData(np.array([start_times, end_times]).T, sections)

    return section_data


def _fix_newpoint(beat_positions):
    """Fills in missing beat position labels by inferring the beat position
    from neighboring beats.

    """
    while np.any(beat_positions == 'New Point'):
        idxs = np.where(beat_positions == 'New Point')[0]
        for i in idxs:
            if i < len(beat_positions) - 1:
                if not beat_positions[i + 1] == 'New Point':
                    beat_positions[i] = str(np.mod(int(beat_positions[i + 1]) - 1, 4))
            if i == len(beat_positions) - 1:
                if not beat_positions[i - 1] == 'New Point':
                    beat_positions[i] = str(np.mod(int(beat_positions[i - 1]) + 1, 4))
    beat_positions[beat_positions == '0'] = '4'

    return beat_positions


def cite():
    """Print the reference"""

    cite_data = """
===========  MLA ===========

Mauch, Matthias, et al.
"OMRAS2 metadata project 2009."
10th International Society for Music Information Retrieval Conference (2009)

========== Bibtex ==========
@inproceedings{mauch2009beatles,
    title={OMRAS2 metadata project 2009},
    author={Mauch, Matthias and Cannam, Chris and Davies, Matthew and Dixon, Simon and Harte,
    Christopher and Kolozali, Sefki and Tidhar, Dan and Sandler, Mark},
    booktitle={12th International Society for Music Information Retrieval Conference},
    year={2009},
    series = {ISMIR}
}
    """

    print(cite_data)
